package net.wanji.datacenter.kafka;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.datacenter.pojo.dto.ImbalanceAlgorithmDTO;
import net.wanji.datacenter.service.CrossStrategyOptService;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/23 15:44
 * @description 神思路口策略优化结果
 */
@Component
@Slf4j
public class CrossStrategyOptConsumerHandler implements KafkaListenerErrorHandler {

    @Resource
    private CrossStrategyOptService crossStrategyOptService;

    /**
     * 相位空放优化指标周期
     *
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.phaseEmptyTopic}"}, groupId = "group")
    public void receivePhaseEmptyData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String message = String.valueOf(record.value());
            if (StringUtils.isNotBlank(message)) {
                crossStrategyOptService.savePhaseEmpty(message);
            }
        } catch (Exception e) {
            log.error("相位空放phaseEmpty数据处理异常；", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    /**
     * 失衡算法结果
     *
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.imbalanceAlgorithmResultTopic}"}, groupId = "group")
    public void receiveImbalanceData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String message = String.valueOf(record.value());
        try {
            if (StringUtils.isNotBlank(message)) {
                ObjectMapper mapper = JacksonUtils.getInstance();
                ImbalanceAlgorithmDTO imbalanceAlgorithmDTO = mapper.readValue(message, ImbalanceAlgorithmDTO.class);
                // 设置3,优化方案
                imbalanceAlgorithmDTO.setRtnType(3);
                crossStrategyOptService.save(imbalanceAlgorithmDTO);
            }
        } catch (Exception e) {
            log.error("当前数据处理异常：{}", message);
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.overflowAlgorithmResultTopic}"}, groupId = "group")
    public void receiveOverFlowData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String message = String.valueOf(record.value());
        if (StringUtils.isNotBlank(message)) {
            ObjectMapper mapper = JacksonUtils.getInstance();
            ImbalanceAlgorithmDTO imbalanceAlgorithmDTO = mapper.readValue(message, ImbalanceAlgorithmDTO.class);
            // 未操作的不存
            // 溢出 rtnType=0 不操作
            if (0 != imbalanceAlgorithmDTO.getRtnType()) {
                crossStrategyOptService.save(imbalanceAlgorithmDTO);
            }
        }
        acknowledgment.acknowledge();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }
}
