package net.wanji.datacenter.util;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.time.Duration;
import java.util.Collections;
import java.util.HashMap;
import java.util.Properties;

@Component
public class TimeBasedKafkaConsumer {

    private static String lightStatusTopic;

    private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

    @Value("${kafka.topics.lightStatusTopic}")
    public void setLightStatusTopic(String lightStatusTopic) {
        TimeBasedKafkaConsumer.lightStatusTopic = lightStatusTopic;
    }

    public static void consumeMessagesAroundTimestamp(String bootstrapServers, String topic, long startTimeStamp,
                                                      long endTimeStamp, String kafkaSavepath, String fileName) {
        Properties props = new Properties();
        props.put("bootstrap.servers", bootstrapServers);
        props.put("group.id", "time-based-consumer");
        props.put("key.deserializer", StringDeserializer.class.getName());
        props.put("value.deserializer", StringDeserializer.class.getName());

        File file = new File(kafkaSavepath + File.separator + fileName);
        try (KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
            BufferedWriter writer = new BufferedWriter(new FileWriter(file, true))) {
            TopicPartition partition = new TopicPartition(topic, 0);
            consumer.assign(Collections.singletonList(partition));

            HashMap<TopicPartition, Long> timestampToSearch = new HashMap<>();
            timestampToSearch.put(partition, startTimeStamp);
            long startOffset = consumer.offsetsForTimes(timestampToSearch).get(partition).offset();

            // 开始消费
            consumer.seek(partition, startOffset);
            boolean keepConsuming = true;
            int emptyPollCount = 0; // 记录空轮询次数的计数器
            int maxEmptyPolls = 10; // 设置最大空轮询次数

            while (keepConsuming) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));
                if (records.isEmpty()) {
                    emptyPollCount++; // 如果没有记录，增加空轮询计数
                    if (emptyPollCount >= maxEmptyPolls) {
                        // 如果达到最大空轮询次数，退出循环
                        break;
                    }
                } else {
                    emptyPollCount = 0; // 如果有记录，重置空轮询计数器
                    for (ConsumerRecord<String, String> record : records) {
                        saveRecordToDisk(record, writer);
                        if (record.timestamp() > endTimeStamp) {
                            keepConsuming = false;
                            break;
                        }
                    }
                }
            }
        } catch (IOException e) {
            System.err.println("Error writing to file: " + e.getMessage());
        }
    }

    private static void saveRecordToDisk(ConsumerRecord<String, String> record, BufferedWriter writer) throws IOException {
        String value = record.value();
        writer.write(value);
        writer.newLine();
        writer.flush();
    }

}
