package com.wanji.indicators.cache;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.wanji.indicators.model.event.WaitingAreaDo;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author fengyi
 * @date 2023/10/31
 * @description
 */
@Slf4j
public class PedestrianDataCache {
    //安全岛 key:crossId_key value：围栏坐标
    private static Map<String, String> safetyIslandMap = new ConcurrentHashMap<>();
    //人行横道
    private static Map<String, String> crossingWalkMap = new ConcurrentHashMap<>();
    //待转区车道面
    private static Map<String, List<WaitingAreaDo>> waitingTurnAreaMap = new ConcurrentHashMap<>();

    private static PedestrianDataCache instance;

    private PedestrianDataCache() {
    }

    public static synchronized PedestrianDataCache getInstance() {
        if (instance == null) {
            instance = new PedestrianDataCache();
            Properties properties = PropertiesHelper.getInstance().getProperties();
            String dataDirectory = properties.getProperty("data.file.path");
            if (StringUtils.isEmpty(dataDirectory)) {
                dataDirectory = "/data/";
            }
            if (!dataDirectory.endsWith("/")) {
                dataDirectory += "/";
            }
            String f1 = dataDirectory + "gis-aqd.geojson";
            String f2 = dataDirectory + "gis-rxhd.geojson";
            String f3 = dataDirectory + "gis-dzq.geojson";
            instance.loadData(f1, 1);
            instance.loadData(f2, 2);
            instance.loadData(f3, 3);
        }
        return instance;
    }

    private Map<String, String> loadData(String file, int type) {
        Map<String, String> retMap = new HashMap<>();

        try (InputStream inputStream = PropertiesHelper.class.getResourceAsStream(file);
             BufferedReader br = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
        ) {
            StringBuilder jsonBuilder = new StringBuilder();
            String str = null;
            // 按行读取字符串
            while ((str = br.readLine()) != null) {
                jsonBuilder.append(str);
            }
            log.info("加载行人区域信息,路径：{},数据：{}", file, jsonBuilder.toString());
            if (jsonBuilder.length() > 0) {
                JSONObject jsonObject = JSON.parseObject(jsonBuilder.toString());
                JSONArray jsonArray = jsonObject.getJSONArray("features");
                int size = jsonArray.size();
                for (int i = 0; i < size; i++) {
                    JSONObject jsonObject1 = jsonArray.getJSONObject(i);
                    JSONObject propJsonObj = jsonObject1.getJSONObject("properties");
                    String crossId = propJsonObj.getString("lkid");
                    String rid = propJsonObj.getString("ldid");
                    String laneId = propJsonObj.getString("cdid");

                    JSONObject geometryJsonObj = jsonObject1.getJSONObject("geometry");
                    JSONArray coordinatesArray = geometryJsonObj.getJSONArray("coordinates");

                    StringBuilder stringBuilder = new StringBuilder();
                    for (int j = 0; j < coordinatesArray.size(); j++) {
                        JSONArray pointArr = coordinatesArray.getJSONArray(j);
                        for (int k = 0; k < pointArr.size(); k++) {
                            double lng = pointArr.getJSONArray(k).getDouble(0);
                            double lat = pointArr.getJSONArray(k).getDouble(1);
                            stringBuilder.append(lng + "," + lat);
                            if (k != pointArr.size() - 1) {
                                stringBuilder.append(";");
                            }
                        }
                    }
                    //Geometry geometry = GeomsConvertUtil.genGeometry(stringBuilder.toString());

                    if (type == 1) {
                        instance.safetyIslandMap.put(crossId + "_" + rid, stringBuilder.toString());
                    } else if (type == 2) {
                        instance.crossingWalkMap.put(rid, stringBuilder.toString());
                    } else if (type == 3) {
                        String waitTurnDir = propJsonObj.getString("cdfx");
                        List<WaitingAreaDo> list = waitingTurnAreaMap.get(rid);
                        if (list == null) {
                            list = new ArrayList<>();
                        }
                        crossId = rid.substring(11,22);
                        WaitingAreaDo model = new WaitingAreaDo();
                        model.setCrossId(crossId);
                        model.setLaneId(laneId);
                        model.setRid(rid);
                        model.setTurnDir(waitTurnDir);
                        model.setGeomWkt(stringBuilder.toString());
                        list.add(model);
                        instance.waitingTurnAreaMap.put(rid, list);
                    }
                }
            }
        } catch (Exception e) {
            log.error("", e);
        }
        return retMap;
    }

    public Map<String, String> getSafetyIslandMap(String crossId) {
        Map<String, String> crossWaitArea = new HashMap<>();
        for (Map.Entry<String, String> entry : safetyIslandMap.entrySet()) {
            String key = entry.getKey();
            if (key.startsWith(crossId)) {
                crossWaitArea.put(key, entry.getValue());
            }
        }
        return crossWaitArea;
    }

    public Map<String, String> getCrossingWalkMap() {
        return crossingWalkMap;
    }

    /**
     * 通过区域相交获取所属rid
     *
     * @param lng
     * @param lat
     * @return
     */
    public String getRidByBmxGeomIntersect(double lng, double lat) {
        for (Map.Entry<String, String> entry : instance.crossingWalkMap.entrySet()) {
            String rid = entry.getKey();
            String areaPoints = entry.getValue();
            boolean isIn = GeomsConvertUtil.withinGeo(lng + "," + lat, areaPoints);
            if (isIn) {
                return rid;
            }
        }
        return null;
    }

    /**
     * 通过区域相交获取所属rid
     *
     * @param lng
     * @param lat
     * @return
     */
    public String getRidByAqdGeomIntersect(double lng, double lat) {
        for (Map.Entry<String, String> entry : instance.safetyIslandMap.entrySet()) {
            String rid = entry.getKey();
            String areaPoints = entry.getValue();
            boolean isIn = GeomsConvertUtil.withinGeo(lng + "," + lat, areaPoints);
            if (isIn) {
                return rid;
            }
        }
        return null;
    }

    public Map<String, List<WaitingAreaDo>> getWaitingTurnAreaMap() {
        return waitingTurnAreaMap;
    }

}
