package com.wanji.indicators.event.cross.conflictpoint.func;

import com.vividsolutions.jts.algorithm.Angle;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.event.confilct.EventCross;
import com.wanji.indicators.model.event.deadlock.CrossDeadLockStatusModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.awt.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 检测是否存在冲突运行的车辆
 */
public class ConflictCheckKeyedProcess extends KeyedProcessFunction<String, CrossFrameModel, EventCross> {

    private static final Logger log = LoggerFactory.getLogger(ConflictCheckKeyedProcess.class);

    private MapState<String, CrossDeadLockStatusModel> mapState;
    private MapState<String, CrossDeadlockResultModel> resultMapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateConflict_stop_1", String.class, CrossDeadLockStatusModel.class));
        resultMapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateConflict_result_1", String.class, CrossDeadlockResultModel.class));
    }

    @Override
    public void processElement(CrossFrameModel deadLockFrameModel, Context context, Collector<EventCross> collector) throws Exception {

        ConfigModel configModel = deadLockFrameModel.getConfigModel();
        long nowTime = deadLockFrameModel.getGlobalTimeStamp();

        List<CarTrackModel> trackList = deadLockFrameModel.getTrackList();

        //key:cross-rid-ridDir-turn value:crossid-rid-ridDir-冲突转向
        Map<String, List<String>> conflictMap = deadLockFrameModel.getConflictSlotMap();
        //按RID分组
        Map<String, List<CarTrackModel>> groupList = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));

        if (groupList.size() >= 2) {
            //====================存放按RID+方向+转向上的的车辆集合============================================//
            Map<String, List<CarTrackModel>> ridTurnCarList = new HashMap<>();
            for (Map.Entry<String, List<CarTrackModel>> entry : groupList.entrySet()) {

                List<CarTrackModel> value = entry.getValue();
                for (CarTrackModel ridCar : value) {
                    String laneFunc = TurnEnum.getLaneFunction(ridCar.getRoadnet().getTurn());
                    String[] sps = laneFunc.split(Constant.COMMA);
                    for (String func : sps) {
                        String ridTurnKey = this.getKey(ridCar, func);
                        List<CarTrackModel> tmplist = ridTurnCarList.get(ridTurnKey);
                        if (Objects.isNull(tmplist)) {
                            tmplist = new ArrayList<>();
                        }
                        tmplist.add(ridCar);
                        ridTurnCarList.put(ridTurnKey, tmplist);
                    }
                }
            }
            //==============================================================================================//
            //速度阈值
            List<CarTrackModel> slowSpeedData = trackList.stream().filter(o -> o.getSpeed() <= 5.0).collect(Collectors.toList());
            List<CarTrackModel> highSpeedData = trackList.stream().filter(o -> o.getSpeed() > 5.0).collect(Collectors.toList());

            //低速目标
            for (CarTrackModel carTrack : slowSpeedData) {

                String[] laneFunc = TurnEnum.getLaneTurnValue(carTrack.getRoadnet().getTurn());
                for (String func : laneFunc) {
                    if (Objects.nonNull(func)) {
                        if (Objects.equals(func,carTrack.getCarTurn().toString())) {
                            String ridTurnKey = this.getKey(carTrack, func);
                            //获取冲突进口及转向
                            List<String> mapList = conflictMap.get(ridTurnKey);
                            if (Objects.nonNull(mapList)) {
                                for (String ckey : mapList) {
                                    //检测冲突方向是否有车辆
                                    List<CarTrackModel> conflictDirCarList = ridTurnCarList.get(ckey);
                                    if (conflictDirCarList != null) {
                                        //冲突方向运行车流
                                        CarTrackModel runCar = conflictDirCarList.get(0);
                                        double distance = GeomsConvertUtil.getDistance(carTrack.getLongitude(), carTrack.getLatitude(), runCar.getLongitude(), runCar.getLatitude());
                                        if (distance < 10) {//10m范围内
                                            outResult(collector, runCar, carTrack);
                                        }
                                    }
                                }
                            }
                        }
                    } else {//行人、非机动车目标可能没有绑定到路网相关数据

                        //视野范围内车辆,从其他方向驶来
                        Polygon polygon = CommonUtil.getCarFrontSector(carTrack);
                        for (CarTrackModel objModel : highSpeedData) {
                            Point point = GeomsConvertUtil.createPoint(objModel.getLongitude() + "," + objModel.getLatitude());
                            if (point.within(polygon)) {
                                int stopDir = CommonUtil.getAngleDir(carTrack.getCourseAngle());
                                int runDir = CommonUtil.getAngleDir(objModel.getCourseAngle());
                                if (stopDir != runDir) {
                                    outResult(collector, carTrack, objModel);
                                }
                            }
                        }
                    }
                }
            }

        }

    }

    private void outResult(Collector<EventCross> collector, CarTrackModel runCar, CarTrackModel stopCar) {
        EventCross run = toEventCross(runCar);

        EventCross stop = toEventCross(stopCar);
        stop.setTrackIdNear(stop.getTrackId() + Constant.COMMA + run.getTrackId());
        stop.setNearTrackObj(run);

        collector.collect(stop);
    }

    public static void main(String[] args) {
        System.out.println(Angle.toDegrees(Angle.diff(129, 110)));
    }


    private String getKey(CarTrackModel carTrack, String func) {
        return carTrack.getRoadnet().getCrossId() +
                Constant.MARK +
                carTrack.getRoadnet().getRid() +
                Constant.MARK +
                carTrack.getRoadnet().getRidDir8() +
                Constant.MARK + func
                ;
    }

    private static EventCross toEventCross(CarTrackModel carTrack) {
        EventCross e = new EventCross();

        e.setCrossId(carTrack.getRoadnet().getCrossId());
        e.setRid(carTrack.getRoadnet().getRid());
        e.setEntryLane(carTrack.getRoadnet().getLaneId());
        e.setLaneFunction(TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn()));
        e.setTrackId(carTrack.getId());
        e.setLng(carTrack.getLongitude());
        e.setLat(carTrack.getLatitude());

        e.setSpeed(carTrack.getSpeed());
        e.setDriveAngle(carTrack.getCourseAngle());
        e.setPlateNumber(carTrack.getPicLicense());
        e.setVehicleType(carTrack.getOriginalType());
        e.setTime(carTrack.getGlobalTimeStamp());

        if (VehicleTypeEnum.isMotorVehicles((short) e.getVehicleType())) {
            e.setCategory(1);
        } else if (VehicleTypeEnum.isNonMotorVehicles((short) e.getVehicleType())) {
            e.setCategory(2);
        } else {
            e.setCategory(3);
        }

        return e;
    }

}
