package com.wanji.indicators.event.cross.deadlock;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.event.cross.deadlock.func.*;
import com.wanji.indicators.model.BaseEventResultModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.event.deadlock.CrossDeadlockResultModel;
import com.wanji.indicators.model.event.deadlock.DeadLockFrameModel;
import com.wanji.indicators.sink.EventFileSink;
import com.wanji.indicators.source.ConflictTurnSource;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;

import java.io.Serializable;
import java.util.Properties;

/**
 * 死锁监测
 *
 * @author fengyi
 */
public class DeadlockEventMainNew implements Serializable {

    private StreamExecutionEnvironment env;
    private PropertiesHelper instance;
    private Properties p;
    private String path;
    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private DeadlockEventMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    private DeadlockEventMainNew(StreamExecutionEnvironment env) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
    }

    public static DeadlockEventMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new DeadlockEventMainNew(env, path, isDebug);
    }

    public static DeadlockEventMainNew init(StreamExecutionEnvironment env) {
        return new DeadlockEventMainNew(env);
    }


    public void run(DataStream<CrossFrameModel> thinningDataStream) {

        SingleOutputStreamOperator<CrossFrameModel> time = thinningDataStream
                .keyBy(CrossFrameModel::getCrossId)
                .countWindow(3)
                .maxBy("time");

        //解析数据
        SingleOutputStreamOperator<DeadLockFrameModel> frameModelFilter = time
                .flatMap(new DeadlockFlatMap())
                .setParallelism(1)
                .name("路口死锁-解析数据");

        //处理RID挂接、车道功能转换
        SingleOutputStreamOperator<DeadLockFrameModel> ridFuncProcess = frameModelFilter
                .keyBy(DeadLockFrameModel::getCrossId)
                .process(new DeadlockCheckDataKeyedProcess())
                .setParallelism(1)
                .name("路口死锁-处理RID挂接、车道功能转换");


        //加载冲突相位信息
        SingleOutputStreamOperator<DeadLockFrameModel> loadConflictSourceProcess = ridFuncProcess
                .connect(env.addSource(new ConflictTurnSource()).broadcast())
                .flatMap(new DeadlockConflictSlotCoFlatMap())
                .setParallelism(1)
                .name("路口死锁-进口转向关联的冲突转向");

        //冲突方向检测
        SingleOutputStreamOperator<DeadLockFrameModel> conflictDireProcess = loadConflictSourceProcess
                .keyBy(DeadLockFrameModel::getCrossId)
                .process(new DeadlockConflictCheckKeyedProcess())
                .setParallelism(1)
                .name("路口死锁-转向冲突线检测");
        //死锁检测
        SingleOutputStreamOperator<CrossDeadlockResultModel> deadlockCheckProcess = conflictDireProcess
                .keyBy(DeadLockFrameModel::getCrossId)
                .process(new DeadlockStopCheckKeyedProcess())
                .setParallelism(1)
                .name("路口死锁-死锁检测");

        //路口死锁-逻辑去重
        SingleOutputStreamOperator<CrossDeadlockResultModel> deadlockDistinctProcess = deadlockCheckProcess
                .keyBy("crossId", "startTime")
                .process(new DeadlockDistinctProcess())
                .setParallelism(1)
                .name("路口死锁-逻辑去重");

        if (isDebug) {
            SingleOutputStreamOperator<BaseEventResultModel> resultOutProcess = deadlockDistinctProcess
                    .flatMap(new FlatMapFunction<CrossDeadlockResultModel, BaseEventResultModel>() {
                        @Override
                        public void flatMap(CrossDeadlockResultModel baseEventResultModel, Collector<BaseEventResultModel> collector) throws Exception {
                            BaseEventResultModel target = new BaseEventResultModel();
                            BeanUtils.copyProperties(baseEventResultModel, target);
                            target.setPictureTime1(baseEventResultModel.getStartTime());
                            target.setPictureTime2(baseEventResultModel.getTime() + baseEventResultModel.getDuration()/2);
                            target.setPictureTime3(baseEventResultModel.getEndTime());
                            collector.collect(target);
                        }
                    })
                    .setParallelism(1)
                    .name("路口死锁-死锁检测");
            resultOutProcess.addSink(new EventFileSink(path)).setParallelism(1).name("路口死锁-结果输出至本地文件");
            resultOutProcess.print("路口死锁-结果").name("路口死锁-结果打印");
        } else {
            //输出json
            SingleOutputStreamOperator<String> jsonOutProcess = deadlockDistinctProcess
                    .flatMap(new FlatMapFunction<CrossDeadlockResultModel, String>() {
                        @Override
                        public void flatMap(CrossDeadlockResultModel baseEventResultModel, Collector<String> collector) throws Exception {
                            BaseEventResultModel target = new BaseEventResultModel();
                            BeanUtils.copyProperties(baseEventResultModel, target);
                            target.setPictureTime1(baseEventResultModel.getStartTime());
                            target.setPictureTime2(baseEventResultModel.getTime() + baseEventResultModel.getDuration()/2);
                            target.setPictureTime3(baseEventResultModel.getEndTime());
                            collector.collect(JSONObject.toJSONString(target));
                        }
                    })
                    .setParallelism(1)
                    .name("路口死锁-死锁检测");
            /**
             * 数据输出 -> kafka
             */
            FlinkKafkaProducer<String> kafkaProducer = new FlinkKafkaProducer<>(
                    p.getProperty("cross.event.data.topic"),
                    new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                    instance.getProducerProperties(),
                    FlinkKafkaProducer.Semantic.NONE);
            jsonOutProcess.addSink(kafkaProducer).setParallelism(1).name("路口死锁-结果输出至kafka");

            jsonOutProcess.print("路口死锁-结果").name("路口死锁-结果打印");
        }

    }



}
