package com.wanji.indicators.task.citybrain.matchresultdata;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.CarPassVO;
import com.wanji.indicators.task.citybrain.vo.TargetInfoVO;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.CarTypeConverter;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * 数据推送城市大脑
 *
 * @author Kent HAN
 * @date 2024/3/27 9:18
 */
@Slf4j
public class MatchResultPassCarDataMain {

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static final String serverIp;

    public static final String serverMac;

    // 城市大脑 URL 路径
    public static final String passCarUrlPath;
    public static final String targetInfoUrlPath;


    static {
        passCarUrlPath = properties.getProperty("city.brain.passcar.info");
        targetInfoUrlPath = properties.getProperty("city.brain.target.info");
        serverIp = properties.getProperty("server.ip");
        serverMac = properties.getProperty("server.mac");
    }

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 获取配置文件中的kafka消费topic
        String sourceTopic = properties.getProperty("consumer.topic");
        String kafkaServerAddress = properties.getProperty("bootstrap.servers");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(sourceTopic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-pass-car-data")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                "city-brain-matchresultdata-data-source");

        SingleOutputStreamOperator<FrameModel> frameModelStream = stream
                .flatMap(new FrameFlatMap())
                .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<CarPassVO> passcarStream = frameModelStream
                .keyBy(FrameModel::getOrgCode)
                .flatMap(new PassCarRichFlatMap())
                .name("计算虚拟线圈过车数据");

        passcarStream.addSink(new SinkFunction<CarPassVO>() {
            @Override
            public void invoke(CarPassVO carPassVO, Context context) {
                HttpUtil.sendData(passCarUrlPath, carPassVO);
            }
        });


        // 启动 Flink 作业
        env.execute("向城市大脑发送-过车数据");
    }
}
