package com.wanji.indicators.task.citybrain.matchresultdata;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.CarPassVO;
import com.wanji.indicators.task.citybrain.vo.TargetInfoVO;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.CarTypeConverter;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * 数据推送城市大脑
 *
 * @author Kent HAN
 * @date 2024/3/27 9:18
 */
@Slf4j
public class MatchResultTrackDataMain {

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static final String serverIp;

    public static final String serverMac;

    // 城市大脑 URL 路径
    public static final String passCarUrlPath;
    public static final String targetInfoUrlPath;


    static {
        passCarUrlPath = properties.getProperty("city.brain.passcar.info");
        targetInfoUrlPath = properties.getProperty("city.brain.target.info");
        serverIp = properties.getProperty("server.ip");
        serverMac = properties.getProperty("server.mac");
    }

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 获取配置文件中的kafka消费topic
        String sourceTopic = properties.getProperty("consumer.topic");
        String kafkaServerAddress = properties.getProperty("bootstrap.servers");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(sourceTopic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-match-result-track-data")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                "city-brain-matchresultdata-data-source");

        SingleOutputStreamOperator<FrameModel> frameModelStream = stream
                .flatMap(new FrameFlatMap())
                .name("轨迹帧数据-JsonToObject");


        frameModelStream.addSink(new SinkFunction<FrameModel>() {
            @Override
            public void invoke(FrameModel frameModel, Context context) throws Exception {

                List<CarTrackModel> participantList = frameModel.getTrackList();
                if(CollectionUtils.isEmpty(participantList)){
                    return;
                }

                List<TargetInfoVO> targetInfoVOList = new ArrayList<>();
                String orgCode = frameModel.getOrgCode();
                Long globalTimeStamp = frameModel.getGlobalTimeStamp();

                for (CarTrackModel participant : participantList) {
                    TargetInfoVO targetInfoVO = new TargetInfoVO();
                    targetInfoVO.setIpAddress(serverIp);
                    targetInfoVO.setMacAddress(serverMac);

                    CarTrackModel.RoadNet roadnet = participant.getRoadnet();
                    Double angle = participant.getCourseAngle();
                    int angleDir = CommonUtil.getAngleDirExt(angle, roadnet.getCrossId());
                    // 转换方向
                    angleDir = CommonUtil.convertCityBrainDir(angleDir);
                    targetInfoVO.setSectionNo(angleDir);

                    String laneId = roadnet.getLaneId();
                    char c = laneId.charAt(laneId.length() - 1);
                    String s = String.valueOf(c);
                    if (CommonUtil.canConvertToInt(s)) {
                        targetInfoVO.setLaneNo(Integer.parseInt(s));
                    }

                    targetInfoVO.setData_time(globalTimeStamp);
                    targetInfoVO.setDeviceNo(orgCode);
                    targetInfoVO.setId(participant.getId());
                    targetInfoVO.setCarType(CarTypeConverter.convertWanjiToBrain(participant.getOriginalType()));
                    targetInfoVO.setCarColor(CarTypeConverter.convertWanjiToBrain(participant.getOriginalColor()));
                    targetInfoVO.setLongitude(participant.getLongitude());
                    targetInfoVO.setLatitude(participant.getLatitude());
                    targetInfoVO.setSpeed(participant.getSpeed());
                    targetInfoVO.setAngle(participant.getCourseAngle());
                    targetInfoVO.setBrand(participant.getPicLicense());

                    targetInfoVOList.add(targetInfoVO);
                }
                //log.info("推送到大脑轨迹数据帧时间：{}", DateUtil.toDateTime(globalTimeStamp,"yyyy-MM-dd HH:mm:ss.SSS"));
                long st = System.currentTimeMillis();
                HttpUtil.sendData(targetInfoUrlPath, targetInfoVOList);
                long et = System.currentTimeMillis();
                log.info("推送到大脑轨迹数据帧时间：{}，耗时：{}ms", DateUtil.toDateTime(globalTimeStamp,"yyyy-MM-dd HH:mm:ss.SSS"),et-st);

            }
        });

        // 启动 Flink 作业
        env.execute("向城市大脑发送-轨迹数据");
    }
}
