package com.wanji.indicators.task.citybrain.matchresultdata;

/**
 * @author fengyi
 * @date 2023/10/30
 * @description
 */

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import com.wanji.indicators.entity.BaseLaneDetector;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.service.BaseLaneDetectorService;
import com.wanji.indicators.service.impl.BaseLaneDetectorServiceImpl;
import com.wanji.indicators.task.citybrain.vo.CarPassVO;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import org.apache.flink.api.common.functions.RichFlatMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.util.Collector;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.List;

public class PassCarRichFlatMap extends RichFlatMapFunction<FrameModel, CarPassVO> {

    BaseLaneDetectorService baseLaneDetectorService;
    private transient List<BaseLaneDetector> baseLaneDetectorList;
    // 线圈状态，key-车辆ID，value-线圈数据
    private transient MapState<Integer, CarPassVO> state;

    @Override
    public void open(Configuration config) throws Exception {
        super.open(config);

        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
        baseLaneDetectorService = beanConf.getBean(BaseLaneDetectorServiceImpl.class);
        baseLaneDetectorList = baseLaneDetectorService.getSenseLaneDetectors();

        MapStateDescriptor<Integer, CarPassVO> descriptor = new MapStateDescriptor<>("car-pass-state", Integer.class, CarPassVO.class);
        state = getRuntimeContext().getMapState(descriptor);
    }

    @Override
    public void flatMap(FrameModel frameModel, Collector<CarPassVO> collector) throws Exception {

        List<CarTrackModel> participantList = frameModel.getTrackList();
        for (CarTrackModel participant : participantList) {
            Integer participantId = participant.getId();
            Double longitude = participant.getLongitude();
            Double latitude = participant.getLatitude();

            // 判断是否在线圈中
            for (BaseLaneDetector detector : baseLaneDetectorList) {
                String geom = detector.getGeom();
                Geometry geometry = GeomsConvertUtil.fromHexWkb(geom);
                geometry = geometry.convexHull();
                Coordinate[] coordinates = geometry.getCoordinates();
                double length = GeomsConvertUtil.getDistance(coordinates[0].x, coordinates[0].y, coordinates[1].x, coordinates[1].y);
                double difLength = 3 - length;
                if (difLength > 0) {
                    geometry = geometry.buffer(GeomsConvertUtil.meterToDu(difLength / 2));
                }

                Point point = GeomsConvertUtil.createPoint(longitude + "," + latitude);
                boolean inDetector = point.intersects(geometry);

                if (inDetector) {
                    // 如果在线圈内，则更新缓存
                    CarPassVO carPassVO = new CarPassVO();
                    carPassVO.setData_time(frameModel.getGlobalTimeStamp());
                    carPassVO.setDeviceNo(detector.getEquipCode());

                    Double angle = participant.getCourseAngle();
                    int angleDir = CommonUtil.getAngleDirExt(angle, participant.getRoadnet().getCrossId());
                    // 转换方向
                    angleDir = CommonUtil.convertCityBrainDir(angleDir);
                    carPassVO.setSectionNo(angleDir);

                    CarTrackModel.RoadNet roadnet = participant.getRoadnet();
                    String laneId = roadnet.getLaneId();
                    char c = laneId.charAt(laneId.length() - 1);
                    String s = String.valueOf(c);
                    if (CommonUtil.canConvertToInt(s)) {
                        carPassVO.setLaneNo(Integer.parseInt(s));
                    }
                    carPassVO.setCoilNo(detector.getEquipCode());
                    carPassVO.setTargetId(participantId);

                    // 更新缓存
                    state.put(participantId, carPassVO);
                } else {
                    // 如果不在线圈内，但是在缓存中，则收集数据
                    CarPassVO carPassVO = state.get(participantId);
                    if (carPassVO != null) {
                        collector.collect(carPassVO);
                        // 清除缓存
                        state.remove(participantId);
                    }
                }
            }
        }
    }
}