package com.wanji.indicators.task.citybrain.snapshot;

import com.wanji.indicators.task.citybrain.util.HttpUtil;
import com.wanji.indicators.task.citybrain.vo.AreaDataVO;
import com.wanji.indicators.task.citybrain.vo.DynamicQueueVO;
import com.wanji.indicators.task.citybrain.vo.StaticQueueVO;
import com.wanji.indicators.task.trajectory.pojo.LaneSnapshotDataDTO;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.List;
import java.util.Properties;

/**
 * @author Kent HAN
 * @date 2024/3/28 10:34
 */
public class FromSnapShotMain {

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();

    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    // 城市大脑 URL 路径
    public static final String urlPathStatic;
    public static final String urlPathDynamic;
    public static final String urlPatharea;

    static {
        urlPathStatic = properties.getProperty("city.brain.static.queue");
        urlPathDynamic = properties.getProperty("city.brain.dynamic.queue");
        urlPatharea = properties.getProperty("city.brain.area.data");
    }

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 获取配置文件中的kafka消费topic
        String sourceTopic = properties.getProperty("lane.snapshot.data");
        String kafkaServerAddress = properties.getProperty("bootstrap.servers");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(sourceTopic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-snapshot")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                "city-brain-dynamic-queue-data-source");

        SingleOutputStreamOperator<LaneSnapshotDataDTO> frameModelStream = stream
                .map(new LaneSnapshotMapFunction())
                .name("车道快照数据-JsonToObject");

        SingleOutputStreamOperator<List<StaticQueueVO>> staticQueueStream = frameModelStream
                .map(new StaticEnrichWithStaticData())
                .name("转换为静态排队数据");

        SingleOutputStreamOperator<List<DynamicQueueVO>> dynamicQueueStream = frameModelStream
                .map(new DynamicEnrichWithStaticData())
                .name("转换为动态排队数据");

        SingleOutputStreamOperator<List<AreaDataVO>> areaDataStream = frameModelStream
                .map(new AreaEnrichWithStaticData())
                .name("转换为区域数据");

        staticQueueStream.addSink(new SinkFunction<List<StaticQueueVO>>() {
            @Override
            public void invoke(List<StaticQueueVO> staticQueueVOList, Context context) {
                HttpUtil.sendData(urlPathStatic, staticQueueVOList);
            }
        });

        dynamicQueueStream.addSink(new SinkFunction<List<DynamicQueueVO>>() {
            @Override
            public void invoke(List<DynamicQueueVO> dynamicQueueVOList, Context context) {
                HttpUtil.sendData(urlPathDynamic, dynamicQueueVOList);
            }
        });

        areaDataStream.addSink(new SinkFunction<List<AreaDataVO>>() {
            @Override
            public void invoke(List<AreaDataVO> areaDataVOList, Context context) {
                HttpUtil.sendData(urlPatharea, areaDataVOList);
            }
        });

        // 启动 Flink 作业
        env.execute("向城市大脑发送静态数据，动态数据，区域数据");
    }

}
