package com.wanji.indicators.task.citybrain.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.task.citybrain.vo.RequestBodyVO;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.asynchttpclient.AsyncHttpClient;
import org.asynchttpclient.BoundRequestBuilder;
import org.asynchttpclient.Dsl;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.Properties;

/**
 * 构造 HTTP 通用请求
 *
 * @author Kent HAN
 * @date 2024/3/26 10:12
 */
@Slf4j
public class HttpUtil {

    private static final ObjectMapper mapper = new ObjectMapper();

    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    // 城市大脑域名
    public static final String domain;
    public static final String appKey;
    public static final String secretKey;
    // 数据来源公司名称
    public static final String from;

    private static final AsyncHttpClient asyncHttpClient = Dsl.asyncHttpClient();


    static {
        domain = properties.getProperty("city.brain.domain");
        appKey = properties.getProperty("city.brain.app.key");
        secretKey = properties.getProperty("city.brain.secret.key");
        from = properties.getProperty("city.brain.from");
    }

    /**
     * 构造通用请求头
     */
    public static BoundRequestBuilder prepareHttpPost(AsyncHttpClient asyncHttpClient, String urlPath) {
        long timestamp = new Date().getTime();
        String signature = makeSignature(timestamp);

        return asyncHttpClient.preparePost(domain + urlPath)
                .setHeader("Content-Type", "application/json;charset=utf-8")
                .setHeader("timestamp", String.valueOf(timestamp))
                .setHeader("appkey", appKey)
                .setHeader("signature", signature);
    }

    private static String makeSignature(long timestamp) {
        try {
            String toHash = secretKey + timestamp;
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] digest = md.digest(toHash.getBytes());
            StringBuilder hexString = new StringBuilder();
            for (byte aDigest : digest) {
                String hex = Integer.toHexString(0xff & aDigest);
                if (hex.length() == 1) {
                    hexString.append('0');
                }
                hexString.append(hex);
            }
            return hexString.substring(1, 11);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 初始化请求体中的 headers
     */
    public static RequestBodyVO createRequestBodyVO(String urlPath) {
        RequestBodyVO requestBodyVO = new RequestBodyVO();
        RequestBodyVO.Header header = new RequestBodyVO.Header();

        // 从 URL 路径中获取最后一个元素作为名称
        String[] parts = urlPath.split("/");
        String name = parts[parts.length - 1];

        header.setName(name);
        header.setFrom(from);
        requestBodyVO.setHeaders(header);

        return requestBodyVO;
    }

    /**
     * 向城市大脑发送数据
     */
    public static void sendData(String urlPath, Object data) {
        try {
            BoundRequestBuilder boundRequestBuilder = HttpUtil.prepareHttpPost(asyncHttpClient, urlPath);
            String jsonBody = buildJsonFromValue(data, urlPath);
            boundRequestBuilder
                    .setBody(jsonBody)
                    .execute()
                    .toCompletableFuture()
                    .whenComplete((response, throwable) -> {
                        if (throwable != null) {
                            log.error("在发送数据时发生错误", throwable);
                        } else {
                            log.info("数据发送至{}，状态{}，响应体: " + response.getResponseBody(),
                                    urlPath, response.getStatusCode());
                        }
                    });
        } catch (Exception e) {
            log.error("发送数据时发生错误", e);
        }
    }

    public static void closeClient() {
        try {
            asyncHttpClient.close();
        } catch (IOException e) {
            log.error("关闭 HTTP 客户端时发生错误", e);
        }
    }

    private static String buildJsonFromValue(Object data, String urlPath) throws JsonProcessingException {
        RequestBodyVO requestBodyVO = HttpUtil.createRequestBodyVO(urlPath);
        String body = mapper.writeValueAsString(data);
        if (body.startsWith("{")) {
            // 单条数据转为列表形式
            body = "[" + body + "]";
        }
        requestBodyVO.setBody(body);
        // 将RequestBodyVO对象转换为JSON字符串
        String jsonString = mapper.writeValueAsString(requestBodyVO);
        // 创建一个只包含一个元素的列表
        return "[" + jsonString + "]";
    }
}
