package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossTurnDataRealtimePO;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/04/16 11:06
 */
@Service
@Slf4j
public class LaneStatisticsReceive2TurnDataMap implements MapFunction<String, String> {
    @Override
    public String map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossTurnDataRealtimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers)
                    && (headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body)) || !body.contains("data_list")) {
                return null;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {});
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String crossId = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {

                        List<LaneStatisticsDetail> leftList = dataList.stream().filter(po -> po.getDwLeftVehNum() > 0).collect(Collectors.toList());
                        List<LaneStatisticsDetail> straightList = dataList.stream().filter(po -> po.getDwStraightVehNum() > 0).collect(Collectors.toList());
                        List<LaneStatisticsDetail> rightList = dataList.stream().filter(po -> po.getDwRightVehNum() > 0).collect(Collectors.toList());

                        CrossTurnDataRealtimePO leftPO = getCrossTurnDataRealtimePO(leftList, dir, "l", dateTime, crossId);
                        CrossTurnDataRealtimePO straightPO = getCrossTurnDataRealtimePO(straightList, dir, "s", dateTime, crossId);
                        CrossTurnDataRealtimePO rightPO = getCrossTurnDataRealtimePO(rightList, dir, "r", dateTime, crossId);
                        results.add(leftPO);
                        results.add(straightPO);
                        results.add(rightPO);
                    }
                }
            }

            Map<String, List<CrossTurnDataRealtimePO>> map = new HashMap<>();
            map.put(Constant.REAL_TIME_TURN_DATA_KEY, results);
            log.error("字符串序列化： " + mapper.writeValueAsString(map));
            return mapper.writeValueAsString(map);
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2TurnDataMap-异常:", e);
            throw new RuntimeException(e);
        }
    }

    private CrossTurnDataRealtimePO getCrossTurnDataRealtimePO(List<LaneStatisticsDetail> dataList, String dir, String turn, Date startTime, String crossId) {
        CrossTurnDataRealtimePO realtimePO = new CrossTurnDataRealtimePO();
        if (!CollectionUtils.isEmpty(dataList)) {
            // 不区分车型机动车总流量
            int volumeSum = 0;
            // 行人流量
            int volumePerson = 0;
            // 非机动车流量
            int volumeCarNon = 0;
            // 小车流量
            int volumeCarSmall = 0;
            // 中车流量
            int volumeCarMiddle = 0;
            // 大车流量
            int volumeCarBig = 0;
            // 平均速度（km/h）
            double speedAvg = 0;
            // 时间占有率（%）
            double occupancy = 0;
            // 平均车头时距（s）
            double headway = 0;
            // 平均车身间距（s）
            double gap = 0;
            // 85位速度（km/h）
            double speed85 = 0;
            // 车流量总和
            int wArrivalFlow = 0;
            // 平均空间占有率
            double fSpaceOccupyRation = 0;
            // 堵塞状态下最大排队长度
            double byQueueLen = 0;
            // 平均延误
            int wDelay = 0;
            // 平均停车次数
            double byStoppingTimes = 0;
            // 左转总流量
            int dwLeftVehNum = 0;
            // 右转总流量
            int dwRightVehNum = 0;
            // 直行总流量
            int dwStraightVehNum = 0;
            for (LaneStatisticsDetail laneStatisticsDetail : dataList) {
                volumeSum += laneStatisticsDetail.getVolumeSum();
                volumePerson += laneStatisticsDetail.getVolumePerson();
                volumeCarNon += laneStatisticsDetail.getVolumeCarNon();
                volumeCarSmall += laneStatisticsDetail.getVolumeCarSmall();
                volumeCarMiddle += laneStatisticsDetail.getVolumeCarMiddle();
                volumeCarBig += laneStatisticsDetail.getVolumeCarBig();
                speedAvg += Double.parseDouble(laneStatisticsDetail.getSpeedAvg());
                occupancy += Double.parseDouble(laneStatisticsDetail.getOccupancy());
                headway += Double.parseDouble(laneStatisticsDetail.getHeadway());
                gap += Double.parseDouble(laneStatisticsDetail.getGap());
                speed85 += Double.parseDouble(laneStatisticsDetail.getSpeed85());
                wArrivalFlow += Integer.parseInt(laneStatisticsDetail.getWArrivalFlow());
                fSpaceOccupyRation += Double.parseDouble(laneStatisticsDetail.getFSpaceOccupyRation());
                Double currrentQueue = Double.parseDouble(laneStatisticsDetail.getByQueueLen());
                byQueueLen = byQueueLen >= currrentQueue ? byQueueLen : currrentQueue;
                wDelay += (int) Double.parseDouble(StringUtils.isBlank(laneStatisticsDetail.getWDelay()) ? "0" : laneStatisticsDetail.getWDelay());
                byStoppingTimes += Double.parseDouble(laneStatisticsDetail.getByStoppingTimes());
                dwLeftVehNum += laneStatisticsDetail.getDwLeftVehNum();
                dwRightVehNum += laneStatisticsDetail.getDwRightVehNum();
                dwStraightVehNum += laneStatisticsDetail.getDwStraightVehNum();
            }
            String id = String.join("_", crossId, dir, turn);
            realtimePO.setId(id);
            realtimePO.setTurnType(turn);
            realtimePO.setInDir(Integer.valueOf(dir));
            realtimePO.setCrossId(crossId);
            realtimePO.setStartTime(startTime);
            realtimePO.setFlow(dwLeftVehNum + dwRightVehNum + dwStraightVehNum);
            realtimePO.setStatus(1);
            // 默认自由流速度60km/h
            if (speedAvg > 0) {
                realtimePO.setTrafficIndex(Math.round(60 / speedAvg / dataList.size() * 100.0) / 100.0);
            } else {
                realtimePO.setTrafficIndex(1.0);
            }
            realtimePO.setSpeed(Math.round(speedAvg / dataList.size() * 100.0) / 100.0);
            realtimePO.setQueueLength(byQueueLen);
            realtimePO.setStopTimes(Math.round(byStoppingTimes / dataList.size() * 100.0) / 100.0);
            realtimePO.setDelayTime((int) Math.round(wDelay / dataList.size() * 100.0) / 100);
            realtimePO.setSturation(1.0);
            //realtimePO.setVehheadDist();
            realtimePO.setVehheadTime(Math.round(headway / dataList.size() * 100.0) / 100.0);
            realtimePO.setQuality(1.0);
            //realtimePO.setNoStopRate();
            realtimePO.setBatchTime((int) startTime.getTime());
            realtimePO.setGmtCreate(new Date());
            realtimePO.setGmtModified(new Date());
        }

        return realtimePO;
    }
}