package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.source.ForeignDeviceInfoSource;
import com.wanji.indicators.task.citybrain.vo.SpilloverEventVO;
import com.wanji.indicators.task.citybrainreceive.event.spillover.SpilloverEventTempReceiveMap;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.List;
import java.util.Properties;

/**
 * @author duanruiming
 * @date 2024/04/10 13:35
 */
@Slf4j
public class LaneStatisticsReceiveMain {
    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static void main(String[] args) throws Exception {

        try {
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

            String statisticsTopic = properties.getProperty("lane.statistics.receive.topic");
            String spilloverTopic = properties.getProperty("lane.spillover.event.receive.topic");

            KafkaSource<String> statisticsSource = getStringKafkaSource(statisticsTopic);
            KafkaSource<String> spilloverSource = getStringKafkaSource(spilloverTopic);

            DataStream<String> statisticsStream = env.fromSource(statisticsSource, WatermarkStrategy.noWatermarks(),
                    "city-brain-laneStatistics-receive-data-source");
            DataStream<String> spilloverStream = env.fromSource(spilloverSource, WatermarkStrategy.noWatermarks(),
                    "city-brain-spillover-receive-data-source");

            dealLaneRealTimeData(statisticsStream, env);
            // 车道数据中不能处理转向，暂不处理 数据不匹配，统计数据没办法计算
            dealTurnRealTimeData(statisticsStream, env);
            dealDirRealTimeData(statisticsStream, env);
            dealCrossRealTimeData(statisticsStream, spilloverStream, env);

            env.execute("接收城市大脑统计数据发送到kafka准备入库");
        } catch (Exception e) {
            log.error("接收城市大脑统计数据数据异常", e);
            throw new RuntimeException(e);
        }
    }

    private static KafkaSource<String> getStringKafkaSource(String topic) {
        // 获取配置文件中的kafka消费topic
        String kafkaServerAddress = properties.getProperty("bootstrap.servers.citybrain.receive");

        return KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(topic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-" + topic + "-receive")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
    }

    private static void dealCrossRealTimeData(DataStream<String> staticticsStream, DataStream<String> spilloverStream, StreamExecutionEnvironment env) throws Exception {

        try {
            SingleOutputStreamOperator<List<SpilloverEventVO>> spilloverVoStream = spilloverStream.map(new SpilloverEventTempReceiveMap());

            SingleOutputStreamOperator<String> laneRealtimeStream = staticticsStream
                    .connect(spilloverVoStream.broadcast())
                    .flatMap(new LaneStatisticsReceive2CrossDataCoFlatMap())
                    .name("接收城市大脑统计数据-JsonToObject");

            SingleOutputStreamOperator<String> crossStream = laneRealtimeStream
                    .connect(env.addSource(new ForeignDeviceInfoSource()).broadcast())
                    .flatMap(new LaneStatisticsReceive2CrossDataCoFlatMapVersion2())
                    .name("接收城市大脑统计数据计算各个方向数据-JsonToObject");

            // 同周期数据一致，发送Kafka，准备入库
            KafkaSink<String> sink = getStringKafkaSink(Constant.REAL_TIME_CROSS_DATA_TOPIC);
            crossStream.sinkTo(sink).name("接收城市大脑统计数据转化路口实时数据发送至kafka").setParallelism(1);
        } catch (Exception e) {
            log.error("接收城市大脑统计数据转化路口实时数据发送至kafka-异常: ", e);
            throw new RuntimeException(e);
        }
    }

    private static void dealDirRealTimeData(DataStream<String> stream, StreamExecutionEnvironment env) throws Exception {
        try {
            SingleOutputStreamOperator<String> laneRealtimeStream = stream
                    .connect(env.addSource(new ForeignDeviceInfoSource()).broadcast())
                    .flatMap(new LaneStatisticsReceive2DirDataMapVersion2())
                    .name("接收城市大脑统计数据-JsonToObject");

            // 同周期数据一致，发送Kafka，准备入库
            KafkaSink<String> sink = getStringKafkaSink(Constant.REAL_TIME_DIR_DATA_TOPIC);
            laneRealtimeStream.sinkTo(sink).name("接收城市大脑统计数据转化方向实时数据发送至kafka").setParallelism(1);
        } catch (Exception e) {
            log.error("接收城市大脑统计数据转化方向实时数据发送至kafka-异常: ", e);
            throw new RuntimeException(e);
        }
    }

    private static void dealTurnRealTimeData(DataStream<String> stream, StreamExecutionEnvironment env) throws Exception {
        try {
            SingleOutputStreamOperator<String> laneRealtimeStream = stream
                    .connect(env.addSource(new ForeignDeviceInfoSource()).broadcast())
                    .flatMap(new LaneStatisticsReceive2TurnDataMapVersion2())
                    .name("接收城市大脑统计数据-JsonToObject");

            // 同周期数据一致，发送Kafka，准备入库
            KafkaSink<String> sink = getStringKafkaSink(Constant.REAL_TIME_TURN_DATA_TOPIC);
            laneRealtimeStream.sinkTo(sink).name("接收城市大脑统计数据转化转向实时数据发送至kafka").setParallelism(1);
        } catch (Exception e) {
            log.error("接收城市大脑统计数据转化转向实时数据发送至kafka-异常：", e);
            throw new RuntimeException(e);
        }
    }

    private static void dealLaneRealTimeData(DataStream<String> stream, StreamExecutionEnvironment env) throws Exception {
        try {
            // todo 添加溢出事件，将溢出状态写入，默认最小溢出指数
            //SingleOutputStreamOperator<String> laneRealtimeStream = stream
            //        .map(new LaneStatisticsReceive2LaneDataMap())
            //        .name("接收城市大脑统计数据-JsonToObject");
            // todo 修改车道入库版本2
            SingleOutputStreamOperator<String> laneRealtimeStream = stream
                    .connect(env.addSource(new ForeignDeviceInfoSource()).broadcast())
                    .flatMap(new LaneStatisticsReceive2LaneDataMapVersion2())
                    .setParallelism(1)
                    .name("车道数据处理路口车道编号");

            // 同周期数据一致，发送Kafka，准备入库
            KafkaSink<String> sink = getStringKafkaSink(Constant.REAL_TIME_LANE_DATA_TOPIC);
            laneRealtimeStream.sinkTo(sink).name("接收城市大脑统计数据转化车道实时数据发送至kafka").setParallelism(1);
        } catch (Exception e) {
            log.error("接收城市大脑统计数据转化车道实时数据发送至kafka-异常：", e);
            throw new RuntimeException(e);
        }
    }

    private static KafkaSink<String> getStringKafkaSink(String realTimeTurnDataTopic) {
        String kafkaServer = properties.getProperty("bootstrap.servers");
        return KafkaSink.<String>builder()
                .setBootstrapServers(kafkaServer)
                .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                        .setTopic(realTimeTurnDataTopic)
                        .setValueSerializationSchema(new SimpleStringSchema())
                        .build())
                .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                .build();
    }
}
