package com.wanji.indicators.task.citybrainreceive.snapshot;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.entity.ForeignDevicePO;
import com.wanji.indicators.mapper.ForeignDeviceMapper;
import com.wanji.indicators.util.CityBrainReceiveUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.SerializableTimestampAssigner;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.flink.streaming.api.TimeCharacteristic;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.RichAllWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingProcessingTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.*;

public class SnapshotReceiveMain {

    private static final Properties properties = PropertiesHelper.getInstance().getProperties();
    private static final PropertiesHelper instance = PropertiesHelper.getInstance();
    private static final ObjectMapper objectMapper = new ObjectMapper();


    public static void main(String[] args) throws Exception {
        String sinkServerAddress = properties.getProperty("bootstrap.servers.citybrain.sink");
        String sinkTopic = properties.getProperty("citybrain.snapshot.sink.topic");
        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        String kafkaServerAddress = properties.getProperty("bootstrap.servers.citybrain.receive");

        // 创建三种类型的数据流
        SingleOutputStreamOperator<QueueReceiveModel> staticQueueModelStream = createModelStream(env, kafkaServerAddress,
                properties.getProperty("static.queue.receive.topic"), "city-brain-static-queue-receive",
                QueueReceiveModel.class);

        SingleOutputStreamOperator<QueueReceiveModel> dynamicQueueModelStream = createModelStream(env, kafkaServerAddress,
                properties.getProperty("dynamic.queue.receive.topic"), "city-brain-dynamic-queue-receive",
                QueueReceiveModel.class);

        SingleOutputStreamOperator<AreaDataReceiveModel> areaDataModelStream = createModelStream(env, kafkaServerAddress,
                properties.getProperty("area.data.receive.topic"), "city-brain-area-data-receive",
                AreaDataReceiveModel.class);

        SingleOutputStreamOperator<SnapshotModel> staticDynamicStream = staticQueueModelStream
                .connect(dynamicQueueModelStream)
                .flatMap(new StaticDynamicFlatMap())
                .returns(TypeInformation.of(SnapshotModel.class));

        SingleOutputStreamOperator<SnapshotModel> snapshotStream = staticDynamicStream
                .connect(areaDataModelStream)
                .flatMap(new QueueAreaFlatMap())
                .returns(TypeInformation.of(SnapshotModel.class));

//        env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime);
        env.setStreamTimeCharacteristic(TimeCharacteristic.ProcessingTime);
        WatermarkStrategy<SnapshotModel> watermarkStrategy = WatermarkStrategy
                .<SnapshotModel>forBoundedOutOfOrderness(Duration.ofMillis(200))
                .withTimestampAssigner(new SerializableTimestampAssigner<SnapshotModel>() {
                    @Override
                    public long extractTimestamp(SnapshotModel element, long recordTimestamp) {
                        try {
                            if (element.getBody() != null && !element.getBody().isEmpty()
                                    && element.getBody().get(0).getData_time() != null) {
                                return element.getBody().get(0).getData_time();
                            } else {
                                return System.currentTimeMillis();
                            }
                        } catch (Exception e) {
                            return System.currentTimeMillis();
                        }
                    }
                });

        SingleOutputStreamOperator<LaneSnapshotDataVO> timeWindowedStream = snapshotStream
//                .assignTimestampsAndWatermarks(watermarkStrategy)
//                .windowAll(TumblingEventTimeWindows.of(Time.seconds(1)))
                .windowAll(TumblingProcessingTimeWindows.of(Time.seconds(1)))
                .apply(new RichAllWindowFunction<SnapshotModel, LaneSnapshotDataVO, TimeWindow>() {
                    ForeignDeviceMapper foreignDeviceMapper;
                    private transient List<ForeignDevicePO> foreignDevicePOList;

                    @Override
                    public void open(Configuration parameters) throws Exception {
                        super.open(parameters);
                        ApplicationContext beanConf = new ClassPathXmlApplicationContext("spring-container.xml");
                        foreignDeviceMapper = beanConf.getBean(ForeignDeviceMapper.class);
                        // 加载静态数据
                        foreignDevicePOList = loadStaticDataFromDatabase();
                    }

                    private List<ForeignDevicePO> loadStaticDataFromDatabase() {
                        return foreignDeviceMapper.selectAllDevices();
                    }

                    @Override
                    public void apply(TimeWindow window, Iterable<SnapshotModel> values,
                                      Collector<LaneSnapshotDataVO> out) {

                        LaneSnapshotDataVO vo = new LaneSnapshotDataVO();
                        vo.setOrgCode("");

                        long windowEnd = window.getEnd();
                        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
                        String formattedDate = dateFormat.format(new Date(windowEnd));
                        vo.setTimeStamp(formattedDate);

                        List<LaneSnapshotDataVO.EventElement> eventList = buildEventList(values, foreignDevicePOList);
                        vo.setEventList(eventList);
                        vo.setLaneNum(eventList.size());

                        out.collect(vo);

                    }
                });

        SingleOutputStreamOperator<String> toJsonStream = timeWindowedStream
                .flatMap(new FlatMapFunction<LaneSnapshotDataVO, String>() {
                    @Override
                    public void flatMap(LaneSnapshotDataVO result, Collector<String> collector) {
                        collector.collect(JSONObject.toJSONString(result, SerializerFeature.WriteMapNullValue));
                    }
                }).setParallelism(1).name("全量车道快照数据转成json字符串");

        FlinkKafkaProducer<String> kafkaProducer = new FlinkKafkaProducer<>(
                sinkServerAddress,
                sinkTopic,
                new SimpleStringSchema());

        toJsonStream.addSink(kafkaProducer)
                .setParallelism(1).name("将全量车道快照数据发送至kafka");

        // 启动 Flink 作业
        env.execute("接收城市大脑数据");
    }

    private static List<LaneSnapshotDataVO.EventElement> buildEventList(Iterable<SnapshotModel> values,
                                                                        List<ForeignDevicePO> foreignDevicePOList) {
        List<LaneSnapshotDataVO.EventElement> res = new ArrayList<>();
        Set<String> laneIdSet = new HashSet<>();

        for (SnapshotModel snapshotModel : values) {
            List<SnapshotModel.Body> bodies = snapshotModel.getBody();
            for (SnapshotModel.Body body : bodies) {
                List<SnapshotModel.Body.LaneInfo> dataList = body.getData_list();
                for (SnapshotModel.Body.LaneInfo laneInfo : dataList) {

                    LaneSnapshotDataVO.EventElement eventElement = new LaneSnapshotDataVO.EventElement();

                    String wanjiLaneId = buildLaneId(laneInfo.getLaneId(), foreignDevicePOList);
                    // 去重
                    if (!laneIdSet.contains(wanjiLaneId)) {
                        laneIdSet.add(wanjiLaneId);
                        eventElement.setLaneId(wanjiLaneId);
                        eventElement.setStaticQueueLength(laneInfo.getStaticQueueLen());
                        eventElement.setDynamicQueueLength(laneInfo.getDynamicQueueLen());
                        eventElement.setStdSpaceHeadway(laneInfo.getPareto());
                        eventElement.setCarNums(laneInfo.getTargetCount());
                        eventElement.setVehicleNumsRatio(laneInfo.getOccupancy());
                        eventElement.setMeanV(laneInfo.getSpeedAvg());
                        eventElement.setHeadCarPosition(laneInfo.getPosHead());
                        eventElement.setStartV(laneInfo.getSpeedHead());
                        eventElement.setTailCarPosition(laneInfo.getPosTail());
                        eventElement.setEndV(laneInfo.getSpeedTail());
                        eventElement.setQueueNums(laneInfo.getQueueCount());
                        eventElement.setTeamHeadDistance(laneInfo.getQueueHead());
                        eventElement.setTeamTailDistance(laneInfo.getQueueTail());
                        eventElement.setMeanSpaceHeadway(0D);
                        eventElement.setVehicleLengthRatio(0D);
                        eventElement.setOverflow(null);

                        res.add(eventElement);
                    }
                }
            }
        }
        return res;
    }

    private static String buildLaneId(String laneId, List<ForeignDevicePO> foreignDevicePOList) {
        return CityBrainReceiveUtil.convertLaneId(laneId, foreignDevicePOList);
    }

    private static <T> SingleOutputStreamOperator<T> createModelStream(
            StreamExecutionEnvironment env, String kafkaServerAddress, String topic,
            String consumerGroupSuffix, Class<T> clazz) {

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(topic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-" + consumerGroupSuffix)
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(
                source, WatermarkStrategy.noWatermarks(), "KafkaSource-" + topic);

        return stream.map(value -> {
            String processedValue = CityBrainReceiveUtil.processJson(value);
            return objectMapper.readValue(processedValue, clazz);
        }).name("JsonToObject-" + topic);
    }
}
