package com.wanji.indicators.task.citybrainreceive.targetinfo;

import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.BaseLocationConverter;
import com.wanji.indicators.util.BodyColorConverter;
import com.wanji.indicators.util.CarTypeConverter;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PlateColorConverter;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeMap;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TargetInfoConvertProcessFunction extends ProcessWindowFunction<TargetInfoReceiveModel, E1FrameMiniVO, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(TargetInfoConvertProcessFunction.class);

    private Properties properties = new Properties();

    // key 为时间戳
    private transient MapState<String, TargetInfoReceiveModel> mapState;

    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
        MapStateDescriptor<String, TargetInfoReceiveModel> descriptor = new MapStateDescriptor<>(
                "trackCacheMapState", String.class, TargetInfoReceiveModel.class);
        mapState = getRuntimeContext().getMapState(descriptor);
    }

    @Override
    public void process(String key, Context context, Iterable<TargetInfoReceiveModel> elements, Collector<E1FrameMiniVO> out) throws Exception {
        long windowStart = context.window().getStart();
        long windowEnd = context.window().getEnd();

        List<TargetInfoReceiveModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        TreeMap<Long,TargetInfoReceiveModel> sortMap = new TreeMap<>();

        Map<String,List<TargetInfoReceiveModel>> group = list.stream().collect(Collectors.groupingBy(o->o.getHeaders().getFrom()));
        Set<String> fromKeys = new HashSet<>();
        for (Map.Entry<String,List<TargetInfoReceiveModel>> entry : group.entrySet()) {
            String from = entry.getKey();
            List<TargetInfoReceiveModel> value = entry.getValue();

            TargetInfoReceiveModel model = value.stream().max(Comparator.comparing(o->o.getDataTime())).get();

            sortMap.put(model.getDataTime(),model);

            fromKeys.add(from);
            mapState.put(from,model);
        }
        Set<String> keys = StreamSupport.stream(mapState.keys().spliterator(), false).collect(Collectors.toSet());

        Set<String> resSet = new HashSet<>();
        resSet.addAll(keys);
        resSet.removeAll(fromKeys);
        for (String from : resSet) {
            TargetInfoReceiveModel agoModel =  mapState.get(from);
            sortMap.put(agoModel.getDataTime(),agoModel);
        }

        long wanjiCount = sortMap.values().stream().filter(o->o.getHeaders().getFrom().equals("wanji")).count();

        log.info("轨迹合并 窗口开始时间：{},截止时间：{}，数据最大帧时间：{},万集轨迹数量：{},总数量：{}",
                DateUtil.toDateTime(windowStart,"yyyy-MM-dd HH:mm:ss.SSS"),
                DateUtil.toDateTime(windowEnd,"yyyy-MM-dd HH:mm:ss.SSS"),
                DateUtil.toDateTime(sortMap.lastKey(),"yyyy-MM-dd HH:mm:ss.SSS"),
                wanjiCount,
                sortMap.size()
        );

        E1FrameMiniVO e1FrameMiniVO = new E1FrameMiniVO();
        Date date = new Date(sortMap.lastKey());
        e1FrameMiniVO.setTimeStamp(date);
        e1FrameMiniVO.setGlobalTimeStamp(sortMap.lastKey());
        List<E1FrameParticipantMiniVO> participantMiniVOList = buildE1FrameParticipant(sortMap.values());
        e1FrameMiniVO.setE1FrameParticipant(participantMiniVOList);
        e1FrameMiniVO.setOrgCode("");
        e1FrameMiniVO.setParticipantNum(participantMiniVOList.size());

        out.collect(e1FrameMiniVO);

    }


    private List<E1FrameParticipantMiniVO> buildE1FrameParticipant(Iterable<TargetInfoReceiveModel> values) {
        List<E1FrameParticipantMiniVO> participantMiniVOList = new ArrayList<>();

        // 将 Iterable 转换为 Stream
        Map<String, TargetInfoReceiveModel> uniqueFromMap = StreamSupport.stream(values.spliterator(), false)
                .collect(Collectors.toMap(
                        model -> model.getHeaders().getFrom(),  // 键映射器：使用 headers.from 作为键
                        model -> model,                        // 值映射器：直接使用 model
                        (existing, replacement) -> existing)); // 合并函数：在键冲突时，保留已存在的元素

        // 转换 Map 的值集合为 List
        ArrayList<TargetInfoReceiveModel> filteredList = new ArrayList<>(uniqueFromMap.values());

        for (TargetInfoReceiveModel value : filteredList) {
            TargetInfoReceiveModel.Header headers = value.getHeaders();
            String from = headers.getFrom();

            List<TargetInfoReceiveModel.Body> body = value.getBody();
            for (TargetInfoReceiveModel.Body paticipant : body) {
                E1FrameParticipantMiniVO vo = new E1FrameParticipantMiniVO();
                vo.setFrom(from);
                vo.setId(paticipant.getId());
                // 协议编号转换
                vo.setOriginalType(CarTypeConverter.convertBrainToWanji(paticipant.getCarType()));
                vo.setOriginalColor(BodyColorConverter.convertBrainToWanji(paticipant.getCarColor()));

                vo.setLongitude(paticipant.getLongitude());
                vo.setLatitude(paticipant.getLatitude());
                Double altitude = paticipant.getAltitude();
                if (altitude != null) {
                    vo.setAltitude(altitude.intValue());
                }
                vo.setSpeed(paticipant.getSpeed());
                vo.setCourseAngle(paticipant.getAngle());
                vo.setPicLicense(paticipant.getBrand());

                // 协议编号转换
                vo.setBaseLocation(BaseLocationConverter.convertBrainToWanji(paticipant.getSectionNo()));
                vo.setLicenseColor(PlateColorConverter.convertBrainToWanji(paticipant.getPlateType()));

                Double trust = paticipant.getTrust();
                if (trust != null) {
                    vo.setConfidence(trust.byteValue());
                }

                vo.setRoadnet(new Roadnet());
                vo.setWaitingTime(-1);
                vo.setStopNum(-1);
                vo.setSource((byte) -1);
                vo.setDetecSource((byte) -1);
                vo.setSourceId(-1);

                participantMiniVOList.add(vo);
            }
        }
        return  participantMiniVOList;
    }

}
