package com.wanji.indicators.task.citybrainreceive.targetinfo;

import com.wanji.indicators.util.BaseLocationConverter;
import com.wanji.indicators.util.BodyColorConverter;
import com.wanji.indicators.util.CarTypeConverter;
import com.wanji.indicators.util.PlateColorConverter;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.CollectionUtil;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author Kent HAN
 * @date 2024/5/11 10:11
 */
public class TargetInfoProcessFunction extends KeyedProcessFunction<String, TargetInfoReceiveModel, E1FrameMiniVO> {

    // key 为时间戳
    private transient MapState<Long, TargetInfoReceiveModel> mapState;

    @Override
    public void open(Configuration parameters) {
        MapStateDescriptor<Long, TargetInfoReceiveModel> descriptor = new MapStateDescriptor<>(
                "mapState", Long.class, TargetInfoReceiveModel.class);
        mapState = getRuntimeContext().getMapState(descriptor);
    }

    @Override
    public void processElement(TargetInfoReceiveModel value,
                               KeyedProcessFunction<String, TargetInfoReceiveModel, E1FrameMiniVO>.Context ctx,
                               Collector<E1FrameMiniVO> out) throws Exception {
        mapState.put(value.getBody().get(0).getData_time(), value);
        // 注册定时器，每2秒钟触发一次
        ctx.timerService().registerProcessingTimeTimer(ctx.timerService().currentProcessingTime() + 2000);
    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<E1FrameMiniVO> out) throws Exception {
        long endTime = timestamp;
        long startTime = endTime - 2000; // 2秒前
        long timeWindow = 100; // 100毫秒窗口

        Map<String, TargetInfoReceiveModel> latestEntries = new HashMap<>();
        // 遍历每个100毫秒的时间窗口，查找该窗口内的数据
        for (long windowStart = startTime; windowStart < endTime; windowStart += timeWindow) {
            long windowEnd = windowStart + timeWindow;

            Iterator<Map.Entry<Long, TargetInfoReceiveModel>> entries = mapState.entries().iterator();
            while (entries.hasNext()) {
                Map.Entry<Long, TargetInfoReceiveModel> entry = entries.next();
                Long entryTime = entry.getKey();
                // 检查entry是否在当前窗口内
                if (entryTime >= windowStart && entryTime <= windowEnd) {
                    TargetInfoReceiveModel model = entry.getValue();
                    String from = model.getHeaders().getFrom();
                    // 只保留来自同一公司的最新数据
                    if (!latestEntries.containsKey(from) ||
                            latestEntries.get(from).getBody().get(0).getData_time()
                                    < model.getBody().get(0).getData_time()) {
                        latestEntries.put(from, model);
                    }
                    entries.remove(); // 清理缓存
                }
            }

            if (!CollectionUtil.isNullOrEmpty(latestEntries)) {
                // 为当前时间窗口创建并输出 E1FrameMiniVO
                E1FrameMiniVO e1FrameMiniVO = new E1FrameMiniVO();
                Date date = new Date(windowStart);
                e1FrameMiniVO.setTimeStamp(date);
                e1FrameMiniVO.setGlobalTimeStamp(windowStart);
                List<E1FrameParticipantMiniVO> participantMiniVOList = buildE1FrameParticipant(latestEntries.values());
                e1FrameMiniVO.setE1FrameParticipant(participantMiniVOList);
                e1FrameMiniVO.setOrgCode("");
                e1FrameMiniVO.setParticipantNum(participantMiniVOList.size());
                out.collect(e1FrameMiniVO);
            }
            // 清空 map 以用于下一个时间窗口
            latestEntries.clear();
        }

        // 清理过期数据
        Iterator<Map.Entry<Long, TargetInfoReceiveModel>> entries = mapState.entries().iterator();
        while (entries.hasNext()) {
            Map.Entry<Long, TargetInfoReceiveModel> entry = entries.next();
            Long entryTime = entry.getKey();
            if (entryTime < timestamp - 5000) {
                entries.remove();
            }
        }

        // 重新注册定时器以持续触发
        ctx.timerService().registerProcessingTimeTimer(timestamp + 2000);
    }

    private List<E1FrameParticipantMiniVO> buildE1FrameParticipant(Iterable<TargetInfoReceiveModel> values) {
        List<E1FrameParticipantMiniVO> participantMiniVOList = new ArrayList<>();

        // 将 Iterable 转换为 Stream
        Map<String, TargetInfoReceiveModel> uniqueFromMap = StreamSupport.stream(values.spliterator(), false)
                .collect(Collectors.toMap(
                        model -> model.getHeaders().getFrom(),  // 键映射器：使用 headers.from 作为键
                        model -> model,                        // 值映射器：直接使用 model
                        (existing, replacement) -> existing)); // 合并函数：在键冲突时，保留已存在的元素

        // 转换 Map 的值集合为 List
        ArrayList<TargetInfoReceiveModel> filteredList = new ArrayList<>(uniqueFromMap.values());

        for (TargetInfoReceiveModel value : filteredList) {
            TargetInfoReceiveModel.Header headers = value.getHeaders();
            String from = headers.getFrom();

            List<TargetInfoReceiveModel.Body> body = value.getBody();
            for (TargetInfoReceiveModel.Body paticipant : body) {
                E1FrameParticipantMiniVO vo = new E1FrameParticipantMiniVO();
                vo.setFrom(from);
                vo.setId(paticipant.getId());
                // 协议编号转换
                vo.setOriginalType(CarTypeConverter.convertBrainToWanji(paticipant.getCarType()));
                vo.setOriginalColor(BodyColorConverter.convertBrainToWanji(paticipant.getCarColor()));

                vo.setLongitude(paticipant.getLongitude());
                vo.setLatitude(paticipant.getLatitude());
                Double altitude = paticipant.getAltitude();
                if (altitude != null) {
                    vo.setAltitude(altitude.intValue());
                }
                vo.setSpeed(paticipant.getSpeed());
                vo.setCourseAngle(paticipant.getAngle());
                vo.setPicLicense(paticipant.getBrand());

                // 协议编号转换
                vo.setBaseLocation(BaseLocationConverter.convertBrainToWanji(paticipant.getSectionNo()));
                vo.setLicenseColor(PlateColorConverter.convertBrainToWanji(paticipant.getPlateType()));

                Double trust = paticipant.getTrust();
                if (trust != null) {
                    vo.setConfidence(trust.byteValue());
                }

                vo.setRoadnet(new Roadnet());
                vo.setWaitingTime(-1);
                vo.setStopNum(-1);
                vo.setSource((byte) -1);
                vo.setDetecSource((byte) -1);
                vo.setSourceId(-1);

                participantMiniVOList.add(vo);
            }
        }

        return participantMiniVOList;
    }
}
