package com.wanji.indicators.task.citybrainreceive.targetinfo;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Properties;

/**
 * 接收城市大脑数据
 *
 * @author Kent HAN
 * @date 2024/3/27 9:18
 */
public class TargetInfoReceiveMain {
    private static final DateTimeFormatter FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS");
    public static final Properties properties = PropertiesHelper.getInstance().getProperties();
    private static final PropertiesHelper instance = PropertiesHelper.getInstance();

    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // 获取配置文件中的kafka消费topic
        String sourceTopic = properties.getProperty("target.info.receive.topic");
        String kafkaServerAddress = properties.getProperty("bootstrap.servers.citybrain.receive");
        String sinkServerAddress = properties.getProperty("bootstrap.servers.citybrain.sink");
        String sinkTopic = properties.getProperty("citybrain.minidata.sink.topic");

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(kafkaServerAddress)
                .setTopics(sourceTopic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-city-brain-targetinfo-receive")
                .setStartingOffsets(OffsetsInitializer.latest())
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();

        DataStream<String> stream = env.fromSource(source, WatermarkStrategy.noWatermarks(),
                "city-brain-targetinfo-receive-data-source");

        // 获取数据流
        SingleOutputStreamOperator<TargetInfoReceiveModel> frameModelStream = stream
                .map(new TargetInfoReceiveMap())
                .name("接收城市大脑轨迹数据-JsonToObject");

        SingleOutputStreamOperator<E1FrameMiniVO> resultStream = frameModelStream
                .keyBy(frame -> "constantKey")
                .process(new TargetInfoProcessFunction());

        SingleOutputStreamOperator<String> toJsonStream = resultStream
                .flatMap(new FlatMapFunction<E1FrameMiniVO, String>() {
                    @Override
                    public void flatMap(E1FrameMiniVO result, Collector<String> collector) {
                        collector.collect(JSONObject.toJSONString(result, SerializerFeature.WriteMapNullValue));
                    }
                }).setParallelism(1).name("全量轨迹数据转成json字符串");

        FlinkKafkaProducer<String> kafkaProducer = new FlinkKafkaProducer<>(
                sinkServerAddress,
                sinkTopic,
                new SimpleStringSchema());

        toJsonStream.addSink(kafkaProducer)
                .setParallelism(1).name("将全量轨迹数据发送至kafka");

        // 启动 Flink 作业
        env.execute("接收城市大脑轨迹数据");
    }



    public static String format(long timestamp) {
        // 将时间戳转换为 LocalDateTime
        LocalDateTime dateTime = LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), ZoneId.systemDefault());
        return FORMATTER.format(dateTime);
    }
}
