package com.wanji.indicators.task.export;

import com.alibaba.fastjson.JSON;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.sink.PartitionFileSink;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import com.wanji.indicators.util.Tools;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class TrackExportToFileMain {
    private static final Logger log = LoggerFactory.getLogger(TrackExportToFileMain.class);


    public static void main(String[] args) {
        //批量轨迹es index,逗号分割：esIndexName,windowTime


        ParameterTool parameter = ParameterTool.fromArgs(args);
        String bootstrapServers = parameter.get("bootstrapServers");
        String topic = parameter.get("topic");
        String startTime = parameter.get("startTime");
        String endTime = parameter.get("endTime");
        String sinkPath = parameter.get("sinkPath");
        //单位分钟，按几分钟一个文件
        Integer partitionTime = parameter.getInt("partitionTime",0);
        //目录
        if (!sinkPath.endsWith("/")) {
            sinkPath += "/";
        }

        Map<Integer, String> timePathIndex = new HashMap<>();

        if (partitionTime > 0) {//指定文件分区
            DateTime start =   DateTime.parse(startTime, DateTimeFormat.forPattern("yyyy_MM_dd_HH_mm_ss"));
            DateTime end =  DateTime.parse(endTime, DateTimeFormat.forPattern("yyyy_MM_dd_HH_mm_ss"));
            while (start.isBefore(end)) {
                int minute = start.getMinuteOfHour();
                int index = minute / partitionTime * partitionTime;
                String timeIndex = start.toString("yyyyMMddHH")+ Tools.fillZeroFront(index+"",2);

                String sdt = start.toString("yyyy_MM_dd_HH_mm_ss");
                start = start.plusMinutes(partitionTime);

                String edt = start.toString("yyyy_MM_dd_HH_mm_ss");

                String filePath = sinkPath + topic + "-" + sdt + "-" + edt + ".json";

                timePathIndex.put(Integer.valueOf(timeIndex), filePath);
            }
        } else {
            sinkPath += topic + "-" + startTime + "-" + endTime + ".json";
            timePathIndex.put(0, sinkPath);
        }

        long startTimestamp = DateUtil.StringToMillis(startTime, "yyyy_MM_dd_HH_mm_ss");
        long endTimestamp = DateUtil.StringToMillis(endTime, "yyyy_MM_dd_HH_mm_ss");

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        KafkaSource<KafkaRecordModel> source = KafkaSource.<KafkaRecordModel>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setBootstrapServers(bootstrapServers)
                .setTopics(topic)
                .setGroupId(properties.getProperty("consumer.group.id") + "-Export")
                // 从大于等于此时间戳开始的偏移量开始
                .setStartingOffsets(OffsetsInitializer.timestamp(startTimestamp))
                // 至此时间戳偏移量结束
                .setBounded(OffsetsInitializer.timestamp(endTimestamp))
                .setDeserializer(KafkaRecordDeserializationSchema.of(new ReadKafkaDeserialization()))

                .build();

        DataStream<KafkaRecordModel> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-export");


        SingleOutputStreamOperator<KafkaRecordModel> frameModelStream =
                stream.filter(new FilterFunction<KafkaRecordModel>() {
                    @Override
                    public boolean filter(KafkaRecordModel value) throws Exception {
                        Long ts = value.getTimestamp();
                        //Long endTs = DateUtil.StringToMillis("2023-03-30 11:40:00:999","yyyy-MM-dd HH:mm:ss:SSS");
                        if (ts <= endTimestamp) {
                            return true;
                        }
                        return false;
                    }
                }).name("过滤出时间范围内数据");

        SingleOutputStreamOperator<KafkaRecordModel> timeOutStream = frameModelStream
                .keyBy(KafkaRecordModel::getTopic)
                .process(new KeyedProcessFunction<String, KafkaRecordModel, KafkaRecordModel>() {
                    private transient MapState<String, Long> dataTimeState;
                    //连续点间隔超时时间，单位分钟
                    private Integer timeOut = 10;

                    @Override
                    public void open(Configuration parameters) throws Exception {
                        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(12))//状态过期时间
                                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                                .cleanupFullSnapshot()
                                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                                .build();

                        MapStateDescriptor mapStateDescriptor = new MapStateDescriptor<>("Kafka_Data_Export",
                                TypeInformation.of(new TypeHint<Tuple3<String, Integer, String>>() {
                                }),
                                TypeInformation.of(new TypeHint<RoutePathCarTrackModel>() {
                                })
                        );
                        mapStateDescriptor.enableTimeToLive(ttlConfig);
                        dataTimeState = getRuntimeContext().getMapState(mapStateDescriptor);
                    }

                    @Override
                    public void processElement(KafkaRecordModel value, Context context, Collector<KafkaRecordModel> out) throws Exception {
                        out.collect(value);

                        int interval = 60 * 1000;
                        long fireTime = context.timerService().currentProcessingTime() + interval;
                        context.timerService().registerProcessingTimeTimer(fireTime);

                        dataTimeState.put(value.getTopic(), context.timerService().currentProcessingTime());

                        //log.info("register process time:" + DateUtil.toDateTime(fireTime, "yyyy-MM-dd HH:mm:ss"));
                    }

                    @Override
                    public void onTimer(long timestamp, OnTimerContext ctx, Collector<KafkaRecordModel> out) throws Exception {

                        Iterator<Map.Entry<String, Long>> iterator = dataTimeState.iterator();
                        while (iterator.hasNext()) {
                            Map.Entry<String, Long> entry = iterator.next();
                            long dataTime = entry.getValue();
                            //log.info("dataTime:"+DateUtil.toDateTime(dataTime,"yyyy-MM-dd HH:mm:ss")+" onTime:" + DateUtil.toDateTime(timestamp, "yyyy-MM-dd HH:mm:ss"));

                            if (timestamp - dataTime > 30 * 1000) {
                                //超时未收到新数据
                                ctx.timerService().deleteProcessingTimeTimer(timestamp);
                                // env.executeAsync().cancel();
                                //System.exit(0);
                            }
                        }
                    }
                });

        SingleOutputStreamOperator<Map<Integer,String>> toJsonStream =
                timeOutStream.flatMap(
                        new FlatMapFunction<KafkaRecordModel, Map<Integer,String>>() {
                            @Override
                            public void flatMap(KafkaRecordModel value, Collector<Map<Integer,String>> out) throws Exception {
                                Long ts = value.getTimestamp();
                                //Long endTs = DateUtil.StringToMillis("2023-03-30 11:40:00:999","yyyy-MM-dd HH:mm:ss:SSS");
//                                                if (ts > endTimestamp) {
//                                                    System.exit(0);
//                                                }
                                log.info("记录时间：{},数据导出范围【{},{}】", DateUtil.toDateTime(ts, "yyyy-MM-dd HH:mm:ss.SSS"), startTime, endTime);

                                String json = JSON.toJSONString(value);

                                Map<Integer, String> dataMap = new HashMap<>();
                                if (partitionTime > 0) {
                                    DateTime dateTime = new DateTime(new Date(ts));
                                    int minute = dateTime.getMinuteOfHour();
                                    int index = minute / partitionTime * partitionTime;
                                    String timeIndex = dateTime.toString("yyyyMMddHH")+ Tools.fillZeroFront(index+"",2);
                                    dataMap.put(Integer.valueOf(timeIndex), json);
                                } else {
                                    dataMap.put(0, json);
                                }
                                out.collect(dataMap);

                            }
                        })
                        .name("转Json输出");


        try {

            toJsonStream.addSink(new PartitionFileSink(timePathIndex));

            env.execute("根据起始时间范围导出数据到文件");
            //env.executeAsync().cancel();
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
