package com.wanji.indicators.task.freeflow.stream;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.freeflow.service.sink.CustomRedisMapper;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.task.freeflow.service.model.FrameMaxSpeedModel;
import com.wanji.indicators.task.freeflow.service.model.MaxSpeedResultModel;
import com.wanji.indicators.task.freeflow.service.func.SpeedAggregateFunction;
import com.wanji.indicators.task.freeflow.service.func.SpeedProcessWindow;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Duration;
import java.util.*;

/**
 * 自由流速度计算
 */
public class FreeFlowSpeedMain {
     private static final Logger log = LoggerFactory.getLogger(FreeFlowSpeedMain.class);

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);

        DateTime currentDateTime = new DateTime();
        //过去一天0点
        DateTime dateTime = currentDateTime.withMillisOfDay(0).plusDays(-1);

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(properties.getProperty("consumer.topic"))
                .setGroupId(properties.getProperty("consumer.group.id") + "_Free_Flow_Speed")
                /* 设置起始偏移量有以下几种情况：
                       1.从指定的位置消费：OffsetsInitializer.offsets(Map< TopicPartition, Long> offsets)
                       2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                       3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                       4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                       5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                      */
                // 从大于等于此时间戳开始的偏移量开始
                .setStartingOffsets(OffsetsInitializer.timestamp(dateTime.getMillis()))

                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))

                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-freespeed");

        SingleOutputStreamOperator<FrameModel> frameModelStream =
                stream
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<FrameMaxSpeedModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new FlatMapFunction<FrameModel, FrameMaxSpeedModel>() {
                            @Override
                            public void flatMap(FrameModel value, Collector<FrameMaxSpeedModel> out) throws Exception {
                                List<CarTrackModel> list = value.getTrackList();

                                String motorObjectType = Constant.MOTOR_TYPES;
                                String[] sps = motorObjectType.split(",");
                                List<String> typeList = Arrays.asList(sps);

                                //取每帧机动车最大速度
                                OptionalDouble doubleStream = list.stream().filter(o->typeList.contains(o.getOriginalType().toString())).mapToDouble(CarTrackModel::getSpeed).max();

                                if (doubleStream.isPresent()) {
                                    double maxSpeed =  doubleStream.getAsDouble();

                                    FrameMaxSpeedModel model = new FrameMaxSpeedModel();
                                    model.setSpeed(maxSpeed);
                                    model.setTimestamp(value.getGlobalTimeStamp());
                                    model.setDate(DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyyMMdd"));

                                    //log.info("Frame MaxSpeed:"+maxSpeed+" "+DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"));
                                    out.collect(model);
                                }
                            }
                        })
                        .setParallelism(1)
                        .name("计算每帧机动车最大速度");

        SingleOutputStreamOperator<MaxSpeedResultModel> freeSpeedStream =  carTrackModelStream
                .assignTimestampsAndWatermarks(
                        WatermarkStrategy.<FrameMaxSpeedModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
                .keyBy(FrameMaxSpeedModel::getDate)
                //.window(TumblingEventTimeWindows.of(Time.days(1), Time.hours(16)))
                .window(TumblingEventTimeWindows.of(Time.days(1)))
                .aggregate(new SpeedAggregateFunction(),new SpeedProcessWindow())
                .name("按天取最大速度-作为自由流速度");

        try {
            //每2次取较大者更新到redis
            //SingleOutputStreamOperator<MaxSpeedResultModel>  maxStream = freeSpeedStream.keyBy(MaxSpeedResultModel::getDate).countWindow(2).maxBy("maxSpeed");
            //maxStream.print();
            freeSpeedStream.addSink( new RedisSink<>(new RedisConfig().getRedisConfig(), new CustomRedisMapper(Constant.AREA_REALTIME_INDICATOR)));

            env.execute("自由流速度计算工作流");

        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
