package com.wanji.indicators.task.lightstatus;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.LightStatusEnum;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.model.TopicLastTimeModel;
import com.wanji.indicators.task.lightstatus.model.VehicleHeadTimeModel;
import com.wanji.indicators.task.queuelength.service.model.MaxQueueResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.HttpUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.joda.time.Interval;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/3/12
 * @description
 */
public class LaneCarHeadTimeProcessFunction extends KeyedProcessFunction<String, CrossFrameModel, VehicleHeadTimeModel>  {
    //车道上离停止线最近的车辆缓存
    private  MapState<String, CarTrackModel> mapState;
    //车道转向绿灯剩余时长缓存
    private MapState<String, Integer> greenRemainTimeState;
    //绿灯总时长
    private MapState<String, Integer> greenTotalTimeState;
    //车道车头时距缓存
    private MapState<String, VehicleHeadTimeModel> resultState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("light_status_map", String.class, CarTrackModel.class));
        greenRemainTimeState = getRuntimeContext().getMapState(new MapStateDescriptor<>("light_time_map", String.class, Integer.class));
        resultState = getRuntimeContext().getMapState(new MapStateDescriptor<>("light_result_map", String.class, VehicleHeadTimeModel.class));
        greenTotalTimeState = getRuntimeContext().getMapState(new MapStateDescriptor<>("light_time_map", String.class, Integer.class));

    }

    @Override
    public void processElement(CrossFrameModel value, Context ctx, Collector<VehicleHeadTimeModel> out) throws Exception {
        long globalTimeStamp = value.getGlobalTimeStamp();

        List<CarTrackModel> trackList = value.getTrackList();
        //灯态数据
        List<CrossRidTurnLampStatusModel> ridLightStatusList = value.getRidTurnLampList();
        //当前绿灯放行方向
        Map<Integer, List<CrossRidTurnLampStatusModel>> groupLightStatus =
                ridLightStatusList.stream()
                        .filter(o -> o.getLampState().equals(LightStatusEnum.GREEN.getType()))
                        .collect(Collectors.groupingBy(o -> o.getDir()));

        for (Map.Entry<Integer, List<CrossRidTurnLampStatusModel>> entry : groupLightStatus.entrySet()) {
            Integer dir = entry.getKey();
            List<CrossRidTurnLampStatusModel> value1 = entry.getValue();

            //绿灯放行进入路口内的车辆
            List<CarTrackModel> runList = trackList.stream()
                    .filter(o -> entry.getKey().equals(o.getRoadnet().getRidDir8()) && o.getRoadnet().getInCrossFlag() == Constant.CROSSING)
                    .collect(Collectors.toList());

            if (!runList.isEmpty()) {
                //按车道分组
                Map<String, List<CarTrackModel>> groupByLane = runList.stream().collect(Collectors.groupingBy(O -> O.getRoadnet().getLaneId()));
                for (Map.Entry<String, List<CarTrackModel>> entry1 : groupByLane.entrySet()) {

                    List<CarTrackModel> laneCarList = entry1.getValue();
                    //计算路口内车辆到停止线距离
                    laneCarList.forEach(o -> {
                        String laneEndPoint = o.getRoadnet().getLaneEndPoint();
                        if (Objects.nonNull(laneEndPoint)) {
                            String[] sps = laneEndPoint.split(",");
                            //距离停止线距离
                            double stopLineDist = GeomsConvertUtil.getDistance(o.getLongitude(), o.getLatitude(), Double.parseDouble(sps[0]), Double.parseDouble(sps[1]));
                            o.setStopLineDist(stopLineDist);
                        }

                    });
                    //按距离排序
                    laneCarList = laneCarList.stream().sorted(Comparator.comparing(o -> o.getStopLineDist())).collect(Collectors.toList());
                    //取靠近停止线的一辆车
                    CarTrackModel lastCar = laneCarList.get(laneCarList.size() - 1);

                    String key = lastCar.getRoadnet().getLaneId();
                    String laneFunction = TurnEnum.getLaneFunction(lastCar.getRoadnet().getTurn());


                    CarTrackModel agoCar = mapState.get(key);
                    Integer agoGreenTime = greenRemainTimeState.get(key);
                    //绿灯区间计算
                    if (agoCar != null && agoGreenTime != null) {
                        //有绿灯时长时监测是否有后车
                        if (agoGreenTime > 0){
                            Integer agoTrackId = agoCar.getId();
                            Integer trackId = lastCar.getId();
                            //新的车辆驶出停止线
                            if (!Objects.equals(agoTrackId, trackId)) {
                                //车辆类型
                                Integer lastCarType = lastCar.getOriginalType();
                                //车长配置，单位米
                                int carLength = 4;
                                //前车车辆类型
                                Integer agoCarType = agoCar.getOriginalType();
                                int agoCarLength = 4;
                                //TODO 不同车型长度配置
                                double distance = PtInPolyUtil.getDistance(agoCar.getLongitude(), agoCar.getLongitude(), lastCar.getLongitude(), lastCar.getLatitude());
                                double lastCarSpeed = lastCar.getSpeed() / 3.6;
                                //计算车头时距(空当时间) （前车车尾到后车车头距离）/后车速度
                                double time = (distance - carLength / 2 - agoCarLength / 2) / lastCarSpeed;

                                VehicleHeadTimeModel agoTimeModel = resultState.get(key);
                                if (agoTimeModel != null) {
                                    //空当时间累加
                                    time = agoTimeModel.getHeadTime() + time;
                                }

                                VehicleHeadTimeModel timeModel = new VehicleHeadTimeModel();
                                timeModel.setLaneId(key);
                                timeModel.setCrossId(lastCar.getRoadnet().getCrossId());
                                timeModel.setRid(lastCar.getRoadnet().getRid());
                                timeModel.setDir(dir);
                                timeModel.setHeadTime(time);
                                timeModel.setTurn(laneFunction);
                                timeModel.setGreenTime(greenTotalTimeState.get(key));

                                resultState.put(key, timeModel);
                                mapState.put(key, lastCar);
                            }
                        } else {
                            //绿灯结束时，输出计算结果
                            List<VehicleHeadTimeModel> list = StreamSupport.stream(resultState.values().spliterator(), false).collect(Collectors.toList());
                            //筛选当前进口、相同转向的 车道
                            list = list.stream().filter(o->o.getRid().equals(lastCar.getRoadnet().getRid()) && o.getTurn().equals(value1.get(0).getTurn())).collect(Collectors.toList());
                            if (!list.isEmpty()) {
                                //所有车道车头时距求平均
                                double avgTime = list.stream().mapToDouble(o -> o.getHeadTime()).summaryStatistics().getAverage();
                                VehicleHeadTimeModel finalOutModel = list.get(0);
                                finalOutModel.setHeadTime(avgTime);
                                out.collect(finalOutModel);
                            }
                            //TODO 输出以后清理缓存
                        }
                    } else {
                        mapState.put(key, lastCar);
                        //只在第一次进入时缓存作为绿灯时长
                        greenTotalTimeState.put(key, value1.get(0).getCyclePhaseCountDown());
                        //相位时长倒计时
                        greenRemainTimeState.put(key, value1.get(0).getCyclePhaseCountDown());
                    }

                }
            }
        }

    }
}
