package com.wanji.indicators.task.monitor.service.func;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.model.TopicLastTimeModel;
import com.wanji.indicators.task.track.service.func.ProcessCarTrackByKey;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.HttpUtil;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.joda.time.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;

/**
 * @author fengyi
 * @date 2023/3/12
 * @description
 */
public class TopicMonitorKeyProcessFunction extends KeyedProcessFunction<String, TopicLastTimeModel, TopicLastTimeModel> {
    private static final Logger log = LoggerFactory.getLogger(TopicMonitorKeyProcessFunction.class);

    private Properties properties;
    private ValueState<TopicLastTimeModel> valueState;

    private int delay;//无数据间隔 单位分钟
    private int timeOut;//数据超时时间 单位分钟

    public TopicMonitorKeyProcessFunction(int delay, int timeOut, Properties properties) {
        this.delay = delay;
        this.timeOut = timeOut;
        this.properties = properties;
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        TypeInformation<TopicLastTimeModel> topicInfo = TypeInformation.of(new TypeHint<TopicLastTimeModel>() {
        });
        valueState = getRuntimeContext().getState(new ValueStateDescriptor<TopicLastTimeModel>("TOPIC_LAST_STATE", topicInfo));
    }

    @Override
    public void processElement(TopicLastTimeModel value, Context ctx, Collector<TopicLastTimeModel> out) throws Exception {
        TopicLastTimeModel currentTopicModel = valueState.value();

        if (currentTopicModel == null) {
            currentTopicModel = new TopicLastTimeModel();
            BeanUtils.copyProperties(value, currentTopicModel);
            currentTopicModel.setBootstrapServers(properties.getProperty("bootstrap.servers"));
            //X分钟定时器
            long coalescedTime = ((ctx.timerService().currentProcessingTime() + delay * 60 * 1000) / 1000) * 1000;
            log.info("Topic："+currentTopicModel.getTopic()+" 数据时间：" + DateUtil.toDateTime(currentTopicModel.getTimestamp(), "yyyy-MM-dd HH:mm:ss.SSS") + "，当前时间：" + DateUtil.toDateTime(ctx.timerService().currentProcessingTime(), "yyyy-MM-dd HH:mm:ss.SSS") + " ，定时器下次触发时间：" + DateUtil.toDateTime(coalescedTime, "yyyy-MM-dd HH:mm:ss.SSS"));
            //注册x分钟定时器
            ctx.timerService().registerProcessingTimeTimer(coalescedTime);
        } else {
            currentTopicModel.setTimestamp(value.getTimestamp());
        }
        valueState.update(currentTopicModel);

        out.collect(currentTopicModel);

    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<TopicLastTimeModel> out) throws Exception {

        TopicLastTimeModel current = valueState.value();

        log.info(ctx.getCurrentKey() + "==>onTimer:" + DateUtil.toDateTime(ctx.timestamp(), "yyyy-MM-dd HH:mm:ss.SSS") + ",数据时间：" + DateUtil.toDateTime(current.getTimestamp(), "yyyy-MM-dd HH:mm:ss.SSS"));

        if (Objects.nonNull(current)) {

            DateTime nowTime = DateTime.now();
            DateTime dataTime = new DateTime(current.getTimestamp());

            long minute = 0;
            if (dataTime.isBeforeNow()) {
                //数据时间与当前时间的时间差
                Interval interval = new Interval(dataTime, nowTime);
                Duration duration = interval.toDuration();
                minute = duration.getStandardMinutes();
            }

//            StringBuilder stringBuilder = new StringBuilder();
//            stringBuilder.append("Kafka全域感知数据检测：\r\n");
//            stringBuilder.append("问题描述：全域感知数据于 " + dataTime.toString("yyyy-MM-dd HH:mm:ss") + " 开始无数据\r\n");
//            stringBuilder.append("Kafka服务地址：" + current.getBootstrapServers() + "\r\n");
//            stringBuilder.append("KafkaTopic：" + current.getTopic() + "\r\n");
//            stringBuilder.append("检测时间：" + nowTime.toString("yyyy-MM-dd HH:mm:ss"));


            JSONObject jsonObject = new JSONObject();
            jsonObject.put("msg_type", "text");

            JSONObject contentObject = new JSONObject();
            contentObject.put("text", "Kafka Topic：" + current.getTopic() + "于" + dataTime.toString("yyyy-MM-dd HH:mm:ss") + "开始无数据");

            jsonObject.put("content", contentObject);

            log.info(jsonObject.toString());

            out.collect(current);

            String at = "<at user_id=\\\\\\\"ou_xxx\\\\\\\">Tom</at>";

            if (minute > this.timeOut) {
                Map<String, String> propertyMap = new HashMap<>();
                propertyMap.put("Content-Type", "application/json");
                String urlStr = properties.getProperty("alarm.feishu.url");
                byte[] bytes = HttpUtil.httpRequest(urlStr, "POST", jsonObject.toJSONString(), "UTF-8", 10, propertyMap);
                if (bytes != null && bytes.length > 0) {
                    String json = new String(bytes, "UTF-8");
                    log.info("飞书告警请求：{}\r\n" + "飞书告警响应：{}", jsonObject.toString(), json);
                }
            }

            long coalescedTime = ((ctx.timerService().currentProcessingTime() + delay * 60 * 1000) / 1000) * 1000;
            ctx.timerService().registerProcessingTimeTimer(coalescedTime);
            log.info("Topic："+current.getTopic()+" 数据时间：" + DateUtil.toDateTime(current.getTimestamp(), "yyyy-MM-dd HH:mm:ss.SSS") + "，" + DateUtil.toDateTime(nowTime.getMillis(), "yyyy-MM-dd HH:mm:ss.SSS") + "，定时器下次触发时间： " + DateUtil.toDateTime(coalescedTime, "yyyy-MM-dd HH:mm:ss.SSS"));

        }
    }
}
