package com.wanji.indicators.task.monitor.stream;

import com.wanji.indicators.model.TopicLastTimeModel;
import com.wanji.indicators.task.monitor.service.func.CustomKafkaDeserialization;
import com.wanji.indicators.task.monitor.service.func.TopicMonitorKeyProcessFunction;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.ProcessingTimeSessionWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.PartitionInfo;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Duration;
import java.util.*;

public class KafkaMonitorMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final Logger log = LoggerFactory.getLogger(TopicMonitorKeyProcessFunction.class);

    public static void main(String[] args) {
        int freqTime = 5;
        int timeOut = 30;

        ParameterTool parameter = ParameterTool.fromArgs(args);
        //topic列表，逗号分割
        String topicList = parameter.get("topicList");
        freqTime = Integer.valueOf(parameter.get("freqTime","5"));
        timeOut = Integer.valueOf(parameter.get("timeOut","5"));

        String[] topicArray = topicList.split(",");

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();
        //构建消费者
        Consumer<String, String> consumer = createConsumer(properties);
        //获取最后消费的topic分区offset
        Map<TopicPartition, Long> endingOffsetMap = getEndingOffsets(consumer,topicArray);

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        //env.setParallelism(2);
        DateTime now = DateTime.now();
        DateTime preDateTime = now.minusHours(24);
        KafkaSource<TopicLastTimeModel> source = KafkaSource.<TopicLastTimeModel>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topicArray)
                .setGroupId(properties.getProperty("consumer.group.id") + "_DATA_MONITOR")
                //.setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.EARLIEST))
                .setStartingOffsets(OffsetsInitializer.offsets(endingOffsetMap))
                .setDeserializer(KafkaRecordDeserializationSchema.of(new CustomKafkaDeserialization()))
                .build();


        DataStream<TopicLastTimeModel> stream = env
                .fromSource(source, WatermarkStrategy.<TopicLastTimeModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                 , "kafka-data-monitor-source");

        SingleOutputStreamOperator<TopicLastTimeModel> frameModelStream =
                stream
                        .keyBy(TopicLastTimeModel::getTopic)
                        .process(new TopicMonitorKeyProcessFunction(freqTime, timeOut, properties))

//                        .window(ProcessingTimeSessionWindows.withGap(Time.minutes(1)))
//                        .process(new ProcessWindowFunction<TopicLastTimeModel, TopicLastTimeModel, String, TimeWindow>() {
//                            @Override
//                            public void process(String s, Context context, Iterable<TopicLastTimeModel> elements, Collector<TopicLastTimeModel> out) throws Exception {
//                                System.out.println("***************************");
//                            }
//                        })
                        .name("Kafka-Topic 数据监控");

        try {
            //frameModelStream.print();
            env.execute("Kafka-Topic数据监控");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }

    @SuppressWarnings("unchecked")
    public static Map<TopicPartition, Long> getEndingOffsets(Consumer<String, String> consumer,String[] topicArray) {
        Map<TopicPartition, Long> endingOffsetMap = new HashMap<>();
        try {
            //topic列表
            Map<String, List<PartitionInfo>> topics = consumer.listTopics();

            for (String topic : topicArray) {
                List<PartitionInfo> partitionInfos = topics.get(topic);
                if (partitionInfos == null) {
                    log.warn("Partition information was not found for topic {}", topic);
                } else {
                    Collection<TopicPartition> partitions = new ArrayList<>();
                    for (PartitionInfo partitionInfo : partitionInfos) {
                        partitions.add(new TopicPartition(topic, partitionInfo.partition()));
                    }
                    //最新offset
                    Map<TopicPartition, Long> endingOffsets = consumer.endOffsets(partitions);
                    Map<TopicPartition, Long> offsetMap = new HashMap<>();
                    for (Map.Entry<TopicPartition, Long> entry : endingOffsets.entrySet()){
                        offsetMap.put(entry.getKey(), entry.getValue()-1);
                    }
                    endingOffsetMap.putAll(offsetMap);
                }
            }
        } finally {
            consumer.close();
        }
        return endingOffsetMap;
    }

    private static Consumer<String, String> createConsumer(Properties properties) {
        final Properties props = new Properties();
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, properties.getProperty("bootstrap.servers"));
        props.put(ConsumerConfig.GROUP_ID_CONFIG, properties.getProperty("consumer.group.id") + "_DATA_MONITOR");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());

        final Consumer<String, String> consumer = new KafkaConsumer<>(props);

        return consumer;
    }

}
