package com.wanji.indicators.task.motor;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;

import java.time.Duration;
import java.util.*;

@Slf4j
public class MotorEfficiencyMainNew {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties properties = new Properties();

    private String path;

    private String indexName;


    private MotorEfficiencyMainNew(StreamExecutionEnvironment env, String indexName) {
        this.env = env;
        this.indexName = indexName;
        this.instance = PropertiesHelper.getInstance();
        this.properties = this.instance.getProperties();
    }


    public static MotorEfficiencyMainNew init(StreamExecutionEnvironment env, String indexName) {
        return new MotorEfficiencyMainNew(env, indexName);
    }

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private MotorEfficiencyMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.properties = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static MotorEfficiencyMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new MotorEfficiencyMainNew(env, path, isDebug);
    }


    public void run(SingleOutputStreamOperator<CrossFrameModel> carTrackModelStream) {

        try {

            SingleOutputStreamOperator<CrossFrameModel> filterStream =
                    carTrackModelStream.flatMap(new FlatMapFunction<CrossFrameModel, CrossFrameModel>() {
                        @Override
                        public void flatMap(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
                            List<CarTrackModel> trackList = value.getTrackList();
                            List<CarTrackModel> trackFilterList = new ArrayList<>();
                            for (CarTrackModel model : trackList) {
                                boolean isMotor = VehicleTypeEnum.isMotorVehicles(model.getOriginalType().shortValue());
                                int inCrossFlag = model.getRoadnet().getInCrossFlag();
                                if (isMotor && inCrossFlag == Constant.CROSSING) {
                                    trackFilterList.add(model);
                                }
                            }
                            value.setTrackList(trackFilterList);
                            if (!trackFilterList.isEmpty()) {
                                out.collect(value);
                            }

                            //log.info("路口ID:{}, 数据帧时间：{}，路口内机动车数量：{}",value.getCrossId(), DateUtil.toDateTime(value.getGlobalTimeStamp(),"yyyy-MM-dd HH:mm:ss.SSS"),trackFilterList.size());
                        }
                    }).setParallelism(1).name("筛选在路口内的机动车");


            SingleOutputStreamOperator<MotorEfficiencyResultModel> processStream = filterStream
                    .assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CrossFrameModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                    .keyBy(o -> o.getCrossId())
                    .window(TumblingEventTimeWindows.of(Time.minutes(5)))
                    .process(new MotorCarProcessWindow())
                    .setParallelism(1).name("不流畅车-减速车计算");


            SingleOutputStreamOperator<String> toJsonStream = processStream.flatMap(new FlatMapFunction<MotorEfficiencyResultModel, String>() {
                @Override
                public void flatMap(MotorEfficiencyResultModel value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            }).setParallelism(1).name("机动车指标转JSON");

            if (!isDebug) {
                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic("analysis.motor.indicators")
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build();

                toJsonStream.sinkTo(sink).name("机动车指标计算-数据发送至kafka").setParallelism(1);
            } else {
                String path = this.path + "analysis.motor.indicators.json";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("事故计算-结果输出至文件");
            }
        } catch (Exception e) {
            log.error("机动车指标计算任务异常 :   " + e);
        }

    }

}
