package com.wanji.indicators.task.queuelength.stream;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.SnapshotIndicatorModel;
import com.wanji.indicators.model.SnapshotInfo;
import com.wanji.indicators.task.freeflow.service.sink.CustomRedisMapper;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.task.queuelength.service.model.MaxQueueResultModel;
import com.wanji.indicators.task.queuelength.service.func.ParseSnapshotInfoFlatMap;
import com.wanji.indicators.task.queuelength.service.func.QueueAggregateFunction;
import com.wanji.indicators.task.queuelength.service.func.QueueProcessWindow;
import com.wanji.indicators.task.queuelength.service.func.SnapshotFlatMap;
import com.wanji.indicators.task.queuelength.service.sink.QueueRedisMapper;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Duration;
import java.util.Properties;

/**
 * 自由流速度计算
 */
public class MaxQueueLengthMain {
     private static final Logger log = LoggerFactory.getLogger(MaxQueueLengthMain.class);

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);

        DateTime currentDateTime = new DateTime();
        //过去一天0点
        DateTime dateTime = currentDateTime.withMillisOfDay(0).plusDays(-1);

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(properties.getProperty("consumer.snapshot.indicator.topic"))
                .setGroupId(properties.getProperty("consumer.group.id") + "_snapshot")
                /* 设置起始偏移量有以下几种情况：
                       1.从指定的位置消费：OffsetsInitializer.offsets(Map< TopicPartition, Long> offsets)
                       2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                       3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                       4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                       5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                      */
                // 从大于等于此时间戳开始的偏移量开始
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))

                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))

                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-queue");

        SingleOutputStreamOperator<SnapshotIndicatorModel> frameModelStream =
                stream
                        .flatMap(new SnapshotFlatMap())
                        .setParallelism(1)
                        .name("每秒快照数据-JsonToObject");

        SingleOutputStreamOperator<SnapshotInfo> carTrackModelStream =
                frameModelStream.
                        flatMap(new ParseSnapshotInfoFlatMap())
                        .setParallelism(1)
                        .name("快照车道数据-FlatMap");

        SingleOutputStreamOperator<MaxQueueResultModel> freeSpeedStream =  carTrackModelStream
                .assignTimestampsAndWatermarks(
                        WatermarkStrategy.<SnapshotInfo>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
                .keyBy(SnapshotInfo::getGlobalId)
                //.window(TumblingEventTimeWindows.of(Time.minutes(2)))
                .window(SlidingEventTimeWindows.of(Time.minutes(2),Time.seconds(1)))
                .aggregate(new QueueAggregateFunction(),new QueueProcessWindow())
                .name("每5秒计算2分钟最大排队长度");

        try {

            freeSpeedStream.addSink( new RedisSink<MaxQueueResultModel>(new RedisConfig().getRedisConfig(), new QueueRedisMapper(Constant.AREA_REALTIME_INDICATOR)));

            env.execute("2分钟最大排队长度计算工作流");

        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
