package com.wanji.indicators.task.track.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.util.*;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;


public class SplitRoutePathFunction extends KeyedProcessFunction<Tuple3<String, Integer, String>, CarTrackModel, RoutePathCarTrackModel> {
    private static final Logger log = LoggerFactory.getLogger(SplitRoutePathFunction.class);
    //路径节点缓存
    private transient MapState<Tuple3<String, Integer, String>, RoutePathCarTrackModel> mapStateOfLineSplit;
    //缓存线路编号
    private transient MapState<Tuple3<String, Integer, String>, Long[]> lineNoState;


    //连续点间隔超时时间，单位分钟
    private Integer timeOut=5;
    @Override
    public void open(Configuration parameters) throws Exception {
        PropertiesHelper instance = PropertiesHelper.getInstance();

        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(6))//状态过期时间
                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                .cleanupFullSnapshot()
                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                .build();

        MapStateDescriptor mapStateDescriptor = new MapStateDescriptor<>("CAR_ROUTE_PATH_CACHE",
                TypeInformation.of(new TypeHint<Tuple3<String, Integer, String>>() {
                }),
                TypeInformation.of(new TypeHint<RoutePathCarTrackModel>() {
                })
        );

        MapStateDescriptor lineNoCacheDescriptor = new MapStateDescriptor<>("CAR_LINE_NO_CACHE",
                TypeInformation.of(new TypeHint<Tuple3<String, Integer, String>>() {
                }),
                TypeInformation.of(new TypeHint<Long[]>() {
                }
                )
        );

        mapStateDescriptor.enableTimeToLive(ttlConfig);
        lineNoCacheDescriptor.enableTimeToLive(ttlConfig);

        mapStateOfLineSplit = getRuntimeContext().getMapState(mapStateDescriptor);
        lineNoState = getRuntimeContext().getMapState(lineNoCacheDescriptor);

    }

    @Override
    public void processElement(CarTrackModel carTrackModel, Context context, Collector<RoutePathCarTrackModel> collector) throws Exception {
        //orgCod_id_plateNo
        Tuple3<String, Integer, String> currentKey = context.getCurrentKey();

        //数据时间戳
        long ts = carTrackModel.getGlobalTimeStamp();
        Long[] agoLineNo = lineNoState.get(currentKey);
        RoutePathCarTrackModel agoTrack = mapStateOfLineSplit.get(currentKey);

        if (agoTrack != null && agoLineNo != null) {

            int agoSeqNo = agoTrack.getSeqNo();
            long agoTs = agoTrack.getPreTimestamp();
            double agoDistance = agoTrack.getRouteLength();

            //连续两个点之间的总长度
            double distance = GeomsConvertUtil.getDistance(agoTrack.getPreLongitude(), agoTrack.getPreLatitude(), carTrackModel.getLongitude(), carTrackModel.getLatitude());
            distance = ArithOfBigDecmial.round(distance, 2);

            agoTrack.setRouteLength(distance + agoDistance);
            agoTrack.setCreateTime(System.currentTimeMillis());
            agoTrack.getRoadNets().add(carTrackModel.getRoadnet());
            agoTrack.setLicenseColor(carTrackModel.getLicenseColor());
            agoTrack.setOriginalColor(carTrackModel.getOriginalColor());

            long dif = ts - agoTs;
            if (dif > 0) {
                //两点之间时间差超过timeOut分钟、或距离大于2公里 生成路径
                if (dif > timeOut * 60 * 1000 || distance > 2000) {
                    //设置终点信息
                    agoTrack.setEndTime(DateUtil.toDateTime(agoTrack.getPreTimestamp(), "yyyy-MM-dd HH:mm:ss.SSS"));
                    //agoTrack.setEndTime(new Date(carTrackModel.getGlobalTimeStamp()));
                    agoTrack.setEndGlobalTimeStamp(agoTrack.getPreTimestamp());
                    agoTrack.setEndLongitude(agoTrack.getPreLongitude());
                    agoTrack.setEndLatitude(agoTrack.getEndLatitude());
                    //agoTrack.setCreateTime(System.currentTimeMillis());
                    agoTrack.getTargetId().add(getTargetId(carTrackModel));

                    agoTrack.setSeqNo(agoSeqNo + 1);
                    agoTrack.setLineNo(agoLineNo[0].intValue() + 1);
                    agoTrack.setFinished(true);
                    agoTrack.getPointList().add(new Double[]{carTrackModel.getLongitude(),carTrackModel.getLatitude()});

                    collector.collect(agoTrack);
                    //路径输出、清理缓存
                    mapStateOfLineSplit.remove(currentKey);
                    lineNoState.put(currentKey,new Long[]{agoTrack.getLineNo().longValue(),ts});
                    log.info(currentKey+"==>路径生成信息：difTime:{}，distance:{}，路径属性:{}",dif,distance,agoTrack);
                    //作为下个路径起点
                    initCarPathStartInfo(currentKey,carTrackModel,agoTrack.getLineNo());
                } else {
                    //路径中间过程节点
                    agoTrack.setPreLongitude(carTrackModel.getLongitude());
                    agoTrack.setPreLatitude(carTrackModel.getLatitude());
                    agoTrack.setPreTimestamp(carTrackModel.getGlobalTimeStamp());
                    agoTrack.setPreDateTime(DateUtil.toDateTime(carTrackModel.getGlobalTimeStamp(), "yyyy-MM-dd HH:mm:ss.SSS"));

                    agoTrack.setSeqNo(agoSeqNo + 1);

                    agoTrack.getTargetId().add(getTargetId(carTrackModel));

                    agoTrack.getPointList().add(new Double[]{carTrackModel.getLongitude(),carTrackModel.getLatitude()});

                    //agoTrack.setCreateTime(System.currentTimeMillis());

                    //更新状态数据
                    mapStateOfLineSplit.put(currentKey, agoTrack);

                    //log.info(currentKey+"===>路径过程数据："+agoTrack);
                }
            }
        } else {
            //初始起点
            initCarPathStartInfo(currentKey,carTrackModel,0);
        }
        //long currentProcessingTime = context.timerService().currentProcessingTime();
        //long fireTime = currentProcessingTime - currentProcessingTime % 60000 + interval;
        //log.info(currentKey + "==>" + DateUtil.toDateTime(currentProcessingTime, "yyyy-MM-dd HH:mm:ss.SSS") + "," + DateUtil.toDateTime(fireTime, "yyyy-MM-dd HH:mm:ss.SSS"));
        //如果注册相同数据的TimeTimer，后面的会将前面的覆盖，即相同的timeTimer只会触发一次
        //context.timerService().registerProcessingTimeTimer(fireTime);
        int interval = timeOut * 60 * 1000;
        long fireTime = context.timestamp() - context.timestamp() % 60000 + interval;
        context.timerService().registerEventTimeTimer(fireTime);
        // 状态缓存清理
        this.clearState();
    }

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<RoutePathCarTrackModel> out) throws Exception {
        log.info(ctx.getCurrentKey() + "==>onTimer:" + DateUtil.toDateTime(ctx.timestamp(), "yyyy-MM-dd HH:mm:ss.SSS") + "," + DateUtil.toDateTime(ctx.timerService().currentProcessingTime(), "yyyy-MM-dd HH:mm:ss.SSS"));

        Iterator<Map.Entry<Tuple3<String, Integer, String>, RoutePathCarTrackModel>> iterator = mapStateOfLineSplit.iterator();
        Set<Tuple3<String, Integer, String>> expireKeys = new HashSet<>();
        //超时未收到新数据，以最后一条记录作为终点形成一条路径
        while (iterator.hasNext()) {
            Map.Entry<Tuple3<String, Integer, String>, RoutePathCarTrackModel> entry = iterator.next();

            RoutePathCarTrackModel model = entry.getValue();

            //设置终点信息
            model.setEndTime(DateUtil.toDateTime(model.getPreTimestamp(), "yyyy-MM-dd HH:mm:ss.SSS"));
            //model.setEndTime(new Date(model.getPreTimestamp()));
            model.setEndGlobalTimeStamp(model.getPreTimestamp());
            model.setEndLongitude(model.getPreLongitude());
            model.setEndLatitude(model.getPreLatitude());
            model.setSeqNo(model.getSeqNo());
            model.setLineNo(model.getLineNo() + 1);
            model.setFinished(true);
            //重置路径编号
            lineNoState.put(ctx.getCurrentKey(),new Long[]{model.getLineNo().longValue(),model.getEndGlobalTimeStamp()});

            expireKeys.add(entry.getKey());

            if (model.getSeqNo() > 1) {//至少两个点输出

                out.collect(model);

                log.info(ctx.getCurrentKey()+"==>超时未收到路径轨迹点，此时以超时时间策略生成路径：timer:{},路径信息:{}",DateUtil.toDateTime(ctx.timestamp(), "yyyy-MM-dd HH:mm:ss.SSS"),model);
            }
        }
        for (Tuple3<String, Integer, String> key : expireKeys) {
            mapStateOfLineSplit.remove(key);
        }
        ctx.timerService().deleteProcessingTimeTimer(timestamp);
    }

    private Map<String, Object> getTargetId(CarTrackModel carTrackModel) {
        Map<String, Object> hashMap = new HashMap<>();
        hashMap.put("orgCode", carTrackModel.getOrgCode());
        hashMap.put("id", carTrackModel.getId());
        return hashMap;
    }

    //路径初始点
    private void initCarPathStartInfo(Tuple3<String, Integer, String> currentKey,CarTrackModel carTrackModel,int lineNo) throws Exception {
        //路网静态数据
        CarTrackModel.RoadNet roadNet = carTrackModel.getRoadnet();

        RoutePathCarTrackModel initModel = new RoutePathCarTrackModel();
        initModel.setLineNo(lineNo);
        initModel.setSeqNo(1);
        initModel.setPicLicense(carTrackModel.getPicLicense());
        initModel.setOriginalType(carTrackModel.getOriginalType());
        initModel.setLicenseColor(carTrackModel.getLicenseColor());
        initModel.setOriginalColor(carTrackModel.getOriginalColor());

        if (roadNet != null) {
            initModel.getRoadNets().add(roadNet);
            initModel.setStartCrossId(roadNet.getCrossId());
            initModel.setStartSegmentId(roadNet.getSegmentId());
            initModel.setStartRid(roadNet.getRid());
            initModel.setStartLaneId(roadNet.getLaneId());
        }

        initModel.setStartTime(DateUtil.toDateTime(carTrackModel.getGlobalTimeStamp(), "yyyy-MM-dd HH:mm:ss.SSS"));
        //initModel.setStartTime(new Date(carTrackModel.getGlobalTimeStamp()));
        initModel.setStartGlobalTimeStamp(carTrackModel.getGlobalTimeStamp());
        initModel.setStartLongitude(carTrackModel.getLongitude());
        initModel.setStartLatitude(carTrackModel.getLatitude());

        initModel.setPreLongitude(carTrackModel.getLongitude());
        initModel.setPreLatitude(carTrackModel.getLatitude());
        initModel.setPreTimestamp(carTrackModel.getGlobalTimeStamp());
        initModel.setPreDateTime(DateUtil.toDateTime(carTrackModel.getGlobalTimeStamp(), "yyyy-MM-dd HH:mm:ss.SSS"));

        initModel.setCreateTime(System.currentTimeMillis());
        Double[] xy = new Double[]{carTrackModel.getLongitude(),carTrackModel.getLatitude()};
        initModel.getPointList().add(xy);

        initModel.getTargetId().add(getTargetId(carTrackModel));


        //缓存路径初始点状态数据
        mapStateOfLineSplit.put(currentKey, initModel);
        lineNoState.put(currentKey,new Long[]{initModel.getLineNo().longValue(),carTrackModel.getGlobalTimeStamp()});
    }


    private void clearState() throws Exception {
        /**************************过期key-start**********************************************/
        Set<Tuple3<String, Integer, String>> expireKeys = new HashSet<>();
        Iterator<Map.Entry<Tuple3<String, Integer, String>, RoutePathCarTrackModel>> iterator =  mapStateOfLineSplit.iterator();
        while (iterator.hasNext()) {
            Map.Entry<Tuple3<String, Integer, String>, RoutePathCarTrackModel> entry = iterator.next();
            RoutePathCarTrackModel model = entry.getValue();
            long timestamp  = model.getCreateTime();
            if (System.currentTimeMillis() - timestamp > 60 * 60 * 1000) {
                //缓存超过
                expireKeys.add(entry.getKey());
                log.info("Car-Route 过期key：{}，缓存开始时间：{},缓存过期时间：{}",entry.getKey(),model.getStartTime(),DateUtil.getNowTime(DateUtil.YYYY_MM_DD_HH_MM_SS));
            }
        }
        if (!expireKeys.isEmpty()) {
            for (Tuple3<String, Integer, String> key : expireKeys) {
                mapStateOfLineSplit.remove(key);
                lineNoState.remove(key);
            }
        }
        /**************************过期key-end**********************************************/
    }


}
