package com.wanji.indicators.task.trafficstate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.task.area.service.model.AreaIndexAnalysis;
import com.wanji.indicators.task.area.service.sink.TravelRedisMapper;
import com.wanji.indicators.task.trafficstate.func.LaneProcessWindow;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.task.trafficstate.sink.TrafficStateRedisMapper;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.AscendingTimestampsWatermarks;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.IterativeStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.streaming.api.functions.windowing.AllWindowFunction;
import org.apache.flink.streaming.api.functions.windowing.ProcessAllWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.GlobalWindows;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.assigners.TumblingProcessingTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.GlobalWindow;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Slf4j
public class LaneTrafficStateMain implements Serializable {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName = "area.realtime.indicator";

    private int windowTime = 10;

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private LaneTrafficStateMain(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }
    public static LaneTrafficStateMain init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new LaneTrafficStateMain(env, path,isDebug);
    }

    public void run(SingleOutputStreamOperator<LanePeriodData> joinLaneStream) {
        try {
            WatermarkStrategy<LanePeriodData> watermarkStrategy
                    = WatermarkStrategy
                    .<LanePeriodData>forGenerator(t -> new AscendingTimestampsWatermarks<LanePeriodData>())
                    .withTimestampAssigner((x, y) -> x.getTimestamp());

            SingleOutputStreamOperator<LaneTrafficStateResult> keyByLaneStream = joinLaneStream
                    .assignTimestampsAndWatermarks(watermarkStrategy)
                    .keyBy(o->o.getLaneId())
                    .window(SlidingEventTimeWindows.of(Time.minutes(10), Time.minutes(1)))
                    .process(new LaneProcessWindow())
                    .setParallelism(1)
                    .name("车道路况计算-按车道分组");

            SingleOutputStreamOperator<List<LaneTrafficStateResult>> listStream = keyByLaneStream
                    .keyBy(o->o.getId()+"_"+ o.getStartTime())
                    .windowAll(TumblingEventTimeWindows.of(Time.seconds(1)))
                    .process(new ProcessAllWindowFunction<LaneTrafficStateResult, List<LaneTrafficStateResult>, TimeWindow>() {
                        @Override
                        public void process(Context context, Iterable<LaneTrafficStateResult> elements, Collector<List<LaneTrafficStateResult>> out) throws Exception {
                            List<LaneTrafficStateResult> list = StreamSupport.stream(elements.spliterator(),false).collect(Collectors.toList());
                            out.collect(list);
                        }
                    })
                    .name("车道路况计算-车道路况数据集合封装");

            SingleOutputStreamOperator<JSONObject> jsonObjectStream = listStream.flatMap(new FlatMapFunction<List<LaneTrafficStateResult>, JSONObject>() {
                 @Override
                public void flatMap(List<LaneTrafficStateResult> value, Collector<JSONObject> out) throws Exception {
                     JSONObject jsonObject = new JSONObject();
                     jsonObject.put("list",value);
                     jsonObject.put("id",value.get(0).getId());
                     log.info("车道路况集合：{}",jsonObject);
                     out.collect(jsonObject);
                }
            }).name("车道路况计算-车道路况集合转JsonObject");

            SingleOutputStreamOperator<String> toJsonStream = jsonObjectStream.flatMap(new FlatMapFunction<JSONObject, String>() {
                @Override
                public void flatMap(JSONObject jsonObject, Collector<String> out) throws Exception {
                    out.collect(jsonObject.toJSONString());
                }
            }).name("车道路况计算-车道路况集合转JSON");

            if (!isDebug) {
                //存储到redis
                jsonObjectStream.addSink(new RedisSink<JSONObject>(new RedisConfig().getRedisConfig(), new TrafficStateRedisMapper(Constant.LANE_TRAFFIC_STATE )));

                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(p.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic("analysis.lane.status")
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build();
                toJsonStream.sinkTo(sink).name("车道路况-数据发送至kafka").setParallelism(1);
            } else {
                String path = this.path + "traffic.state";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("车道交通路况-结果输出至文件");
            }
        } catch (Exception e) {
            log.error("车道交通路况计算异常",e);
        }
    }


}
