package com.wanji.indicators.task.trafficstate;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.event.stream.func.RefBaseRidInfoCoFlatMap;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.source.AreaFreeSpeedSource;
import com.wanji.indicators.task.area.service.func.RidCarProcessWindow;
import com.wanji.indicators.task.area.service.func.RidFreeSpeedCoFlatMap;
import com.wanji.indicators.task.area.service.func.RidOutProcessWindow;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisOutputModel;
import com.wanji.indicators.task.trafficstate.func.BoundCarRoadNetInfoCoFlatMap;
import com.wanji.indicators.task.trafficstate.func.LaneProcessWindow;
import com.wanji.indicators.task.trafficstate.func.RidMotorCarProcessWindow;
import com.wanji.indicators.task.trafficstate.func.RidProcessWindow;
import com.wanji.indicators.task.trafficstate.model.LanePeriodData;
import com.wanji.indicators.task.trafficstate.model.LaneTrafficStateResult;
import com.wanji.indicators.task.trafficstate.sink.TrafficStateRedisMapper;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.AscendingTimestampsWatermarks;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.windowing.ProcessAllWindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.flink.util.Collector;

import java.io.Serializable;
import java.time.Duration;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Slf4j
public class RidTrafficStateMain implements Serializable {
    private   StreamExecutionEnvironment env;

    private   PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private int windowTime = 10;

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private RidTrafficStateMain(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }

    public static RidTrafficStateMain init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new RidTrafficStateMain(env, path,isDebug);
    }

    public void run(SingleOutputStreamOperator<CarTrackModel> filterStream) {

        try {

            SingleOutputStreamOperator<CarTrackModel> motorStream = filterStream
                    .filter(o-> VehicleTypeEnum.isMotorVehicles(o.getOriginalType().shortValue()))
                    .name("路段交通路况-筛选机动车数据");

            SingleOutputStreamOperator<CarTrackModel> joinLaneStream = motorStream
                    .flatMap(new BoundCarRoadNetInfoCoFlatMap())
                    .setParallelism(1)
                    .name("路段交通路况-车道扩展属性数据绑定");

            SingleOutputStreamOperator<LaneTrafficStateResult> groupByRidStream =
                    joinLaneStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(5))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                            //.keyBy(o->(o.getRoadnet().getSegmentId()))
                            //.process(new RidMotorCarProcessWindow())
                            .keyBy(o->o.getRoadnet().getRid())
                            .window(SlidingEventTimeWindows.of(Time.minutes(1), Time.seconds(1)))
                            .process(new RidProcessWindow())
                            .name("路段交通路况-按RID渠化段分组车辆");

            SingleOutputStreamOperator<List<LaneTrafficStateResult>> listStream = groupByRidStream
                    .keyBy(o->o.getId()+"_"+ o.getStartTime())
                    .windowAll(TumblingEventTimeWindows.of(Time.seconds(1)))
                    .process(new ProcessAllWindowFunction<LaneTrafficStateResult, List<LaneTrafficStateResult>, TimeWindow>() {
                        @Override
                        public void process(Context context, Iterable<LaneTrafficStateResult> elements, Collector<List<LaneTrafficStateResult>> out) throws Exception {
                            List<LaneTrafficStateResult> list = StreamSupport.stream(elements.spliterator(),false).collect(Collectors.toList());
                            out.collect(list);
                        }
                    })
                    .name("路段路况计算-RID路况数据集合封装");

            SingleOutputStreamOperator<JSONObject> jsonObjectStream = listStream.flatMap(new FlatMapFunction<List<LaneTrafficStateResult>, JSONObject>() {
                @Override
                public void flatMap(List<LaneTrafficStateResult> value, Collector<JSONObject> out) throws Exception {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("list",value);
                    jsonObject.put("id",value.get(0).getId());
                    log.info("路段路况集合：{}",jsonObject);
                    out.collect(jsonObject);
                }
            }).name("路段路况计算-车道路况集合转JsonObject");

            SingleOutputStreamOperator<String> toJsonStream = listStream.flatMap(new FlatMapFunction<List<LaneTrafficStateResult>, String>() {
                @Override
                public void flatMap(List<LaneTrafficStateResult> value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            }).name("路段路况计算-转换为json");

            if (!isDebug) {
                //存储到redis
                jsonObjectStream
                        .addSink(new RedisSink<JSONObject>(new RedisConfig().getRedisConfig(), new TrafficStateRedisMapper(Constant.RID_TRAFFIC_STATE )));

                KafkaSink<String> sink = KafkaSink.<String>builder()
                        .setBootstrapServers(p.getProperty("bootstrap.servers"))
                        .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                                .setTopic("analysis.road.segment.status")
                                .setValueSerializationSchema(new SimpleStringSchema())
                                .build()
                        )
                        .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                        .build();
                toJsonStream.sinkTo(sink).name("路段路况-数据发送至kafka").setParallelism(1);
            } else {
                String path = this.path + "rid.traffic.state";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("路段路况计算-结果输出至文件");

            }
        } catch (Exception e) {
            log.error("路段路况计算",e);
        }
    }


}
