package com.wanji.indicators.task.turnIndicators.func;

import com.wanji.indicators.constant.TimeGranularityEnum;
import com.wanji.indicators.entity.BaseCrossTurnInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.task.beihang.BaseGisLaneCache;
import com.wanji.indicators.task.turnIndicators.model.CrossTurnResultModel;
import com.wanji.indicators.task.turnIndicators.model.TurnResultModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeHint;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/4/6
 * @description
 */
public class TurnDataProcessWindowByAngle extends ProcessWindowFunction<CrossFrameModel, CrossTurnResultModel, String, TimeWindow> {

    private static final Logger log = LoggerFactory.getLogger(TurnDataProcessWindowByAngle.class);

    private transient MapState<String, Long> distinctIdState;

    @Override
    public void open(Configuration parameters) throws Exception {
        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(12))//状态过期时间
                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                .cleanupFullSnapshot()
                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                .build();
        MapStateDescriptor mapStateDescriptor = new MapStateDescriptor<>("turn_distinct_id",String.class,Long.class);
        mapStateDescriptor.enableTimeToLive(ttlConfig);
        distinctIdState = getRuntimeContext().getMapState(mapStateDescriptor);
    }

    @Override
    public void process(String groupKey, Context context, Iterable<CrossFrameModel> elements, Collector<CrossTurnResultModel> out) throws Exception {
        List<CrossFrameModel> list = StreamSupport.stream(elements.spliterator(), false).collect(Collectors.toList());

        List<CarTrackModel> trackList = new ArrayList<>();
        for (CrossFrameModel crossFrameModel : list) {
            trackList.addAll(crossFrameModel.getTrackList());
        }
        //按车道分组
        Map<String, List<CarTrackModel>> groupById = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getLaneId()));
        //=====================================================================================//
        long windowStartTime = context.window().getStart();
        long windowEndTime = context.window().getEnd();
        String startTime = DateUtil.toDateTime(windowStartTime, DateUtil.YYYY_MM_DD_HH_MM_SS);
        String endTime = DateUtil.toDateTime(windowEndTime, DateUtil.YYYY_MM_DD_HH_MM_SS);

        long globalTime = windowEndTime;

        Map<String, TurnResultModel> turnResultModelMap = new HashMap<>();
        //按转向计算
        for (Map.Entry<String, List<CarTrackModel>> entry : groupById.entrySet()) {

            List<CarTrackModel> value = entry.getValue();
            //按车辆ID分组
            Map<Integer, List<CarTrackModel>> groupByTrackId = value.stream().collect(Collectors.groupingBy(o -> o.getId()));

            for (Map.Entry<Integer, List<CarTrackModel>> entry1 : groupByTrackId.entrySet()) {
                List<CarTrackModel> value1 = entry1.getValue();
                //按时间排序
                value1 = value1.stream().sorted(Comparator.comparing(o -> o.getGlobalTimeStamp())).collect(Collectors.toList());

                CarTrackModel carTrackModel = value1.get(value1.size()-1);
                double laneLength = carTrackModel.getRoadnet().getLaneLength();
                String laneStartPoint = carTrackModel.getRoadnet().getLaneStartPoint();
                String carPoint = carTrackModel.getLongitude() + "," + carTrackModel.getLatitude();
                double distance = GeomsConvertUtil.getDistance(laneStartPoint, carPoint);

                String laneTurnKey = entry.getKey()+"_"+entry1.getKey();
                Long agoTime = distinctIdState.get(laneTurnKey);
                //监测最后一辆车是否驶过停止线
                if (distance >= (laneLength + 1) && !distinctIdState.contains(laneTurnKey)) {
                    String key = carTrackModel.getRoadnet().getRid()+"_"+carTrackModel.getCarTurn();

                    TurnResultModel resultModel = turnResultModelMap.get(key);
                    if (resultModel == null) {
                        resultModel = new TurnResultModel();
                    }
                    resultModel.setInRid(carTrackModel.getRoadnet().getRid());
                    resultModel.setInRidDir(carTrackModel.getRoadnet().getRidDir8());
                    resultModel.setTurnDirNo(carTrackModel.getCarTurn());
                    //转向流量累加
                    resultModel.setFlow(resultModel.getFlow()+1);
                    turnResultModelMap.put(key, resultModel);
                    //key:laneid_carid
                    distinctIdState.put(laneTurnKey,carTrackModel.getGlobalTimeStamp());
                }

            }

        }
        //输出转向流量
        List<TurnResultModel> outList = new ArrayList<>(turnResultModelMap.values());
        if (!list.isEmpty()) {
            CrossTurnResultModel crossTurnResultModel = new CrossTurnResultModel();
            crossTurnResultModel.setStartTime(startTime);
            crossTurnResultModel.setEndTime(endTime);
            crossTurnResultModel.setTimestamp(context.window().getStart());
            crossTurnResultModel.setCrossId(groupKey);
            crossTurnResultModel.setTurnDataList(outList);
            crossTurnResultModel.setGranularity(TimeGranularityEnum.FIVE_MINUTE.getType());

            String name = BaseGisLaneCache.getInstance().getCrossNameRefMap().get(groupKey);
            crossTurnResultModel.setCrossName(name);

            out.collect(crossTurnResultModel);
        }
        //状态过期清理
        Set<String> expireKeys = new HashSet<>();
        Iterator<Map.Entry<String,Long>> iterator =  distinctIdState.iterator();
        while (iterator.hasNext()) {
            Map.Entry<String,Long> entry = iterator.next();
            long timestamp  = entry.getValue();
            if (globalTime - timestamp > 5 * 60 * 1000) {
                //缓存超过
                expireKeys.add(entry.getKey());
            }
        }
        if (!expireKeys.isEmpty()) {
            for (String key : expireKeys) {
                distinctIdState.remove(key);
            }
        }
    }
}
