package com.wanji.indicators.util;

import com.google.gson.Gson;
import com.wanji.indicators.model.SingleCarTrackListModel;
import org.apache.commons.codec.binary.Base64;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkBase;
import org.apache.flink.streaming.connectors.elasticsearch.ElasticsearchSinkFunction;
import org.apache.flink.streaming.connectors.elasticsearch.util.RetryRejectedExecutionFailureHandler;
import org.apache.flink.streaming.connectors.elasticsearch7.ElasticsearchSink;
import org.apache.flink.streaming.connectors.elasticsearch7.RestClientFactory;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.message.BasicHeader;
import org.elasticsearch.action.admin.indices.mapping.put.PutMappingRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.client.*;
import org.elasticsearch.client.indices.CreateIndexRequest;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentType;
//import sun.misc.BASE64Encoder;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.UUID;

/**
 * @author fengyi
 * @date 2023/3/2
 * @description
 */
public class ElasticSearchSinkUtil {

    private static RestHighLevelClient client = null;
    private static Gson gson = new Gson().newBuilder().create();
    //private static BASE64Encoder base64Encoder = new BASE64Encoder();
    private PropertiesHelper instance;

    static {
        Properties p = PropertiesHelper.getInstance().getProperties();
        List<HttpHost> hostLit = null;
        try {
            hostLit = getEsAddresses(p.getProperty("elasticsearch.server"));
            HttpHost[] hostArray = hostLit.toArray(new HttpHost[hostLit.size()]);
            RestClientBuilder builder = RestClient.builder(hostArray);
            //开始设置用户名和密码
            CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
            credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(p.getProperty("elasticsearch.username"), p.getProperty("elasticsearch.password")));
            builder.setHttpClientConfigCallback(f -> f.setDefaultCredentialsProvider(credentialsProvider));

            ElasticSearchSinkUtil.client = new RestHighLevelClient(builder);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }

    public static RestHighLevelClient getClient() {
        return client;
    }

    public static void close() {
        try {
            ElasticSearchSinkUtil.client.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static boolean isIndexExists(String indexName) {
        try {
            return ElasticSearchSinkUtil.client.indices().exists(new GetIndexRequest(indexName), RequestOptions.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    public static void createIndex(String indexName,boolean isCreateMapping,XContentBuilder mappingBuilder) {
        if (isIndexExists(indexName)) {
            return;
        }
        try {
            CreateIndexRequest index = new CreateIndexRequest(indexName);
            if (isCreateMapping) {
                index.mapping(mappingBuilder);
            }
            //设置嵌套窗口
            index.settings(Settings.builder()
                    .put("index.max_inner_result_window",2000000000)
                     );

            ElasticSearchSinkUtil.client.indices().create(index, RequestOptions.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static <T> IndexResponse add(T obj, String indexName) {
        IndexRequest indexRequest = new IndexRequest(indexName).id(UUID.randomUUID().toString());
        indexRequest.source(new Gson().toJson(obj), XContentType.JSON);

        try {
            return ElasticSearchSinkUtil.client.index(indexRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 解析配置文件的 es hosts
     *
     * @param hosts
     * @return
     * @throws MalformedURLException
     */
    public static List<HttpHost> getEsAddresses(String hosts) throws MalformedURLException {
        String[] hostList = hosts.split(",");
        List<HttpHost> addresses = new ArrayList<>();
        for (String host : hostList) {
            if (host.startsWith("http")) {
                URL url = new URL(host);
                addresses.add(new HttpHost(url.getHost(), url.getPort()));
            } else {
                String[] parts = host.split(":", 2);
                if (parts.length > 1) {
                    addresses.add(new HttpHost(parts[0], Integer.parseInt(parts[1])));
                } else {
                    throw new MalformedURLException("invalid elasticsearch hosts format");
                }
            }
        }
        return addresses;
    }

    /**
     * es sink
     *
     * @param hosts       es hosts
     * @param parallelism 并行数
     * @param data        数据
     * @param func
     * @param <T>
     */
    public static <T> void addSink(String sinkName,List<HttpHost> hosts, int parallelism,
                                   SingleOutputStreamOperator<T> data, ElasticsearchSinkFunction<T> func) {
        ElasticsearchSink.Builder<T> esBuilder = new ElasticsearchSink.Builder<>(hosts, func);

        //设置用户名密码
        esBuilder.setRestClientFactory(restClientFactory);
        // 每100条数据执行一次或5秒执行一次
        esBuilder.setBulkFlushMaxActions(100);
        esBuilder.setBulkFlushInterval(5000);
        // 设置失败重试的次数
        esBuilder.setBulkFlushBackoffRetries(3);
        // 设置重试的时间间隔
        esBuilder.setBulkFlushBackoffDelay(2);
        // 设置重试策略
        esBuilder.setBulkFlushBackoffType(ElasticsearchSinkBase.FlushBackoffType.EXPONENTIAL);
        // 设置失败处理
        esBuilder.setFailureHandler(new RetryRejectedExecutionFailureHandler());

        data.addSink(esBuilder.build()).setParallelism(parallelism).name(sinkName);
    }

    public static RestClientFactory restClientFactory = new RestClientFactory() {
        @Override
        public void configureRestClientBuilder(RestClientBuilder restClientBuilder) {
            PropertiesHelper instance = PropertiesHelper.getInstance();
            String user = instance.getElasticProperties().getProperty("elasticsearch.username");
            String pwd = instance.getElasticProperties().getProperty("elasticsearch.password");
            String auth = Base64.encodeBase64String((user + ":" + pwd).getBytes());
            restClientBuilder.setDefaultHeaders(new BasicHeader[]{new BasicHeader("Authorization", "Basic " + auth)});
        }
    };


}
