package com.wanji.indicators.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;
import java.util.*;

public class JdbcUtil {
  private static final Logger log = LoggerFactory.getLogger(JdbcUtil.class);
  
  private Connection conn;
  
  private PreparedStatement stmt;
  
  private ResultSet rs;
  
  private String driver;
  
  private String url;
  
  private String userName;
  
  private String passWord;
  
  private static volatile JdbcUtil instance = null;
  
  public static JdbcUtil getInstance() {
    if (instance == null)
      synchronized (JdbcUtil.class) {
        if (instance == null)
          instance = new JdbcUtil(); 
      }  
    return instance;
  }
  
  public JdbcUtil() {
    Properties prop = new Properties();
    try {
      prop.load(JdbcUtil.class.getClassLoader().getResourceAsStream("config.properties"));
      this.driver = prop.getProperty("jdbc.driver");
      String postgres_host = System.getenv("POSTGRES_HOST");
      String postgres_port = System.getenv("POSTGRES_PORT");
      String postgres_db = System.getenv("POSTGRES_DB");
      String postgres_user = System.getenv("POSTGRES_USER");
      String postgres_password = System.getenv("POSTGRES_PASSWORD");
      String postgres_schema = System.getenv("POSTGRES_SCHEMA");
      if (StringUtils.isNotBlank(postgres_host) && 
        StringUtils.isNotBlank(postgres_port) && 
        StringUtils.isNotBlank(postgres_db) && 
        StringUtils.isNotBlank(postgres_user) && 
        StringUtils.isNotBlank(postgres_password)) {
        if (StringUtils.isBlank(postgres_schema))
          postgres_schema = "public"; 
        this.url = "jdbc:postgresql://" + postgres_host + ":" + postgres_port + "/" + postgres_db + "?currentSchema=" + postgres_schema;
        this.userName = postgres_user;
        this.passWord = postgres_password;
      } else {
        this.url = prop.getProperty("jdbc.url");
        this.userName = prop.getProperty("jdbc.username");
        this.passWord = prop.getProperty("jdbc.password");
      } 
    } catch (Exception e) {
      log.error("数据库链接异常: " + e.getMessage());
    } 
  }
  
  public Connection getConn() {
    try {
      Class.forName(this.driver);
      this.conn = DriverManager.getConnection(this.url, this.userName, this.passWord);
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
      return null;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    } 
    return this.conn;
  }
  
  public void close() {
    try {
      if (this.rs != null) {
        this.rs.close();
        this.rs = null;
      } 
      if (this.stmt != null) {
        this.stmt.close();
        this.stmt = null;
      } 
      if (this.conn != null) {
        this.conn.close();
        this.conn = null;
      } 
      log.info("数据库连接资源释放 ==>>  rs:" + this.rs + "  stmt:" + this.stmt + "  conn:" + this.conn);
    } catch (SQLException e) {
      log.error("资源释放发生异常: " + e.getMessage());
    } 
  }
  
  public ResultSet executeSql(String sql, Object... args) {
    try {
      getConn();
      this.stmt = this.conn.prepareStatement(sql);
      if (null != args && args.length != 0)
        for (int i = 0; i < args.length; i++)
          this.stmt.setObject(i + 1, args[i]);  
      this.rs = this.stmt.executeQuery();
    } catch (SQLException e) {
      log.error("数据查询异常: " + e.getMessage());
      return null;
    } 
    return this.rs;
  }
  
  private <T> T excuteQuery(Class<T> klass, String sql, Object... args) {
    try {
      this.rs = executeSql(sql, args);
      ResultSetMetaData metaData = this.rs.getMetaData();
      Map<String, Object> resultMap = new HashMap<>();
      if (this.rs.next())
        for (int i = 1; i <= metaData.getColumnCount(); i++) {
          String columnname = metaData.getColumnLabel(i);
          Object obj = this.rs.getObject(i);
          resultMap.put(columnname, obj);
        }  
      return (T)JSON.parseObject(JSON.toJSONString(resultMap), klass);
    } catch (Exception e) {
      log.error("数据查询异常" + e.getMessage());
    } finally {
      close();
    } 
    return (T)JSON.toJavaObject((JSON)new JSONObject(), klass);
  }
  
  public <T> List<T> excuteQueryToList(Class<T> klass, String sql, Object... args) {
    try {
      this.rs = executeSql(sql, args);
      List<Map<String, String>> resultList = new ArrayList<>();
      while (this.rs != null && this.rs.next()) {
        Map<String, String> resultMap = new HashMap<>();
        ResultSetMetaData metaData = this.rs.getMetaData();
        int columnCount = metaData.getColumnCount();
        for (int i = 1; i <= columnCount; i++)
          resultMap.put(metaData.getColumnName(i), this.rs.getString(i)); 
        resultList.add(resultMap);
      } 
      log.info("\n数据库查询成功:  " + sql + "  \n查询到数据: " + resultList.size() + "条  ");
      return JSON.parseArray(JSON.toJSONString(resultList), klass);
    } catch (Exception e) {
      log.error("数据查询异常   " + e.getMessage() + "  time:  " + System.currentTimeMillis());
    } finally {
      close();
    } 
    return null;
  }
}
